<?php
/**
 * Functions for debugging
 * @since 2.0.0
 * @package Bookings for WooCommerce
 */

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Add some debug options
 * @since 2.0.0
 */
function bfwc_debug_tools( $debug_tools ) {

	$debug_tools['bfwc_clean_dates'] = array(
		'name'     => __( 'Clean corrupted Bookings for WooCommerce dates', 'bfwc' ),
		'button'   => __( 'Clean', 'woocommerce' ),
		'desc'     => __( 'This tool will clean any corrupted booking dates.', 'bfwc' ),
		'callback' => 'bfwc_clean_all_dates',
	);

	return $debug_tools;

}
add_filter( 'woocommerce_debug_tools', 'bfwc_debug_tools' );

/**
 * Use this to reset availability for all dates that have become corrupted
 * @since	2.0.0
 * @version	2.0.4
 */
function bfwc_clean_all_dates() {

	// Query all booked days
	$args = array(
		'post_type'			=> 'bfwc_booked_days',
		'post_status'		=> 'publish',
		'posts_per_page'	=> -1,
		'orderby'         	=> 'title',
		'order'           	=> 'DESC',
		'fields'			=> 'ids'
	);

	$booked_days = new WP_Query( $args );
	if( $booked_days->posts ) {
		foreach( $booked_days->posts as $date_id ) {
			bfwc_clean_up_date( $date_id, true );
    	}
  	}

}

/**
 * Use this to reset availability for dates that have become corrupted
 * @since	2.0.0
 * @version	2.0.4
 */
function bfwc_clean_up_date( $date_id, $loop=false ) {

	if ( empty( $date_id ) || ! $loop ) {
		if( ! isset( $_GET['post'] ) ) {
			return;
		} else {
			$date_id = absint( $_GET['post'] );
		}

		if( ! isset( $_GET['clean_date'] ) ) {
			return;
		}

		if( ! isset( $_GET['_wpnonce'] ) || ! wp_verify_nonce( $_GET['_wpnonce'], 'bfwc_clean_date' ) ) {
			return;
		}

		if( ! current_user_can( 'manage_options' ) ) {
			return;
		}
	}

	// Get the date's post record
	$the_date = get_post( $date_id );
	if ( empty( $the_date ) || 'bfwc_booked_days' !== $the_date->post_type ) {
		return;
	}

	// Get the booking meta
	$booking_meta = get_post_meta( $date_id, 'booking_meta', true );

	if( $booking_meta ) {
		
		foreach( $booking_meta as $product_id=>$data ) {

			if( empty( $data['booking_ids'] ) ) {
				unset( $booking_meta[$product_id] );
			} else if ( is_array( $data['booking_ids'] ) ) {

				// 2.0.4, loop through the booking IDs
				$tmp_booking_ids = $tmp_used_by_booking = array();
				foreach ( $data['booking_ids'] as $meta_booking_id ) {

					if ( empty( $meta_booking_id ) ) {
						// perhaps this is the product's non standard cost, or blocked by an external calendar, check
						if ( ! empty( $data['used_by_booking'][0] ) ) {
							$product_post_meta = get_post_meta( $product_id );
							if ( $product_post_meta ) {
								foreach ( $product_post_meta as $key => $value ) {
									if ( '_nonstd_data' === $key && is_serialized( $value[0] ) ) {
										// check if the date is inside the non-standard data?
										$nonstd_data = unserialize( $value[0] );
										if ( is_array( $nonstd_data ) ) {
											// loop through the non standard dates and find our booked day
											foreach ( $nonstd_data as $nsd ) {
												if ( ! empty( $nsd['start_date_ymd'] ) && ! empty( $nsd['end_date_ymd'] ) && $the_date->post_title >= $nsd['start_date_ymd'] && $the_date->post_title <= $nsd['end_date_ymd'] ) {
													// valid
													$tmp_booking_ids[] = 0;
													$tmp_used_by_booking[0] = $data['used_by_booking'][0];
													break; // no need to continue, this is enough?
												}
											}
										}
									} else if ( false !== strpos( $key, 'bfwc_already_blocked_' ) && is_serialized( $value[0]) ) {
										// check if the date is blocked by an external calendar
										$already_blocked = unserialize( $value[0] );
										if ( is_array( $already_blocked ) ) {
											// loop through the blocked dates and find our booked day
											foreach ( $already_blocked as $blocked_date ) {
												if ( $blocked_date === $the_date->post_title ) {
													// valid
													$tmp_booking_ids[] = 0;
													$tmp_used_by_booking[0] = $data['used_by_booking'][0];
													break 2; // stop 2 levels of foreach
												}
											}
										}
									}
								}
							}
						}
						continue; // skip to the next booking ID
					}

					// get this booking's post meta and do some validations
					$booking_post_meta = get_post_meta( $meta_booking_id );
					if ( empty( $booking_post_meta ) || empty( $booking_post_meta['bfwc_product_id'][0] ) || $product_id != $booking_post_meta['bfwc_product_id'][0] ) {
						// do not include this
						continue;
					}

					// get this booking's dates, their formats should be yyyy-mm-dd
					$start_date = $booking_post_meta['bfwc_start_date'][0];
					$end_date = $booking_post_meta['bfwc_end_date_plus_buffer'][0];

					if ( empty( $start_date ) || empty( $end_date ) ) {
						// something's wrong, do not include?
						continue;
					}

					if ( $the_date->post_title >= $start_date && $the_date->post_title <= $end_date && ! in_array( $meta_booking_id, $tmp_booking_ids ) ) {
						// this date is within the booking's start date and end date (including buffer), so this is valid
						$tmp_booking_ids[] = $meta_booking_id;
						$tmp_used_by_booking[$meta_booking_id] = $data['used_by_booking'][$meta_booking_id];
					}

				} // end of foreach $data['booking_ids']

				if ( ! empty( $tmp_booking_ids ) && ! empty( $tmp_used_by_booking ) ) {
					// we have valid data, save this back
					$booking_meta[$product_id]['booking_ids'] = $tmp_booking_ids;
					$booking_meta[$product_id]['used_by_booking'] = $tmp_used_by_booking;

					// then let's recount used_by_booking so that we have an accurate count of used bookings
					$used_bookings = 0;
					foreach ( $booking_meta[$product_id]['used_by_booking'] as $booking_id => $booking_count ) {
						$used_bookings += $booking_count;
					}
					// The used number of bookings can never be more than the max
					$max_bookings = (int)bfwc_get_max_bookings( $product_id );
					$used_bookings = min( $max_bookings, $used_bookings );
					$booking_meta[$product_id]['used_bookings'] = $used_bookings;
				} else {
					// we don't have a valid booking for this product, unset
					unset( $booking_meta[$product_id] );
				}
			}

		}

		update_post_meta( $date_id, 'booking_meta', $booking_meta );

	}

}
add_action( 'admin_init', 'bfwc_clean_up_date' );

/**
 * Make the Booked Days post type public
 * @since 2.0.0
 */
function bfwc_view_booked_days() {
	$view = get_option( 'bfwc_view_booked_days', 'no' );
	if( $view != 'yes' ) {
		return false;
	}
	return true;
}