<?php
/**
 * Functions for importing and exporting calendars
 * @since 2.0.0
 * @package Bookings for WooCommerce
 */

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Create submenu for external services
 */
function bfwc_external_page() {

	add_submenu_page(
		'edit.php?post_type=bfwc_booking',
		__( 'External', 'pewc' ),
		__( 'External', 'pewc' ),
		apply_filters( 'bfwc_external_page_capability', 'manage_options' ),
		'bfwc_external_page',
		'bfwc_external_page_callback'
	);

}
add_action( 'admin_menu', 'bfwc_external_page', 110 );

function bfwc_external_page_callback() { ?>

	<div class="wrap">

		<?php printf( '<h1>%s</h1>', __( 'External services', 'bfwc' ) ); ?>
		<p><?php _e( 'Sync all your calendars for external services like Airbnb.', 'bfwc' ); ?></p>

		<div class="bfwc-sync-cals-wrapper">

			<?php $url = esc_url( add_query_arg( array(
				'sync_cal'		=> 'true',
				'product_id'	=> 'all'
			) ) );
			printf(
				'<p><a class="button" href="%s">%s</a></p>',
				$url,
				__( 'Sync all calendars', 'bfwc' )
			); ?>

		</div>
	</div>

<?php }

/**
 * Create the calendar file
 * @since 2.0.0
 */
function bfwc_create_calendar( $product_id ) {

	// Get the path and filename
	$path = bfwc_get_calendar_path( $product_id );

	// Get the contents of the calendar
	$contents = bfwc_get_calendar_contents( $product_id );

	// Save the file
	file_put_contents( $path, $contents );

}

/**
 * Get the calendar content, blocking out dates that are booked
 * @since 2.0.0
 */
function bfwc_get_calendar_contents( $product_id ) {

	// Get all the bookings for this product
	$booked_days = bfwc_get_bookings_by_id( $product_id );

	if( $booked_days ) {

		$icalobj = new ZCiCal();
		// Time when calendar is created. Format: "Ymd\THis\Z"
		$datestamp = ZCiCal::fromUnixDateTime() . 'Z';

		$title = __( 'Booking from website', 'bfwc' );

		foreach( $booked_days as $booked_day ) {

			$eventobj = new ZCiCalNode("VEVENT", $icalobj->curnode);

			// add title
			$eventobj->addNode( new ZCiCalDataNode( "SUMMARY:" . $title ) );

			// add start date
			$eventobj->addNode( new ZCiCalDataNode( "DTSTART:" . ZCiCal::fromSqlDateTime( $booked_day . " 06:00:00" ) ) );

			// add end date
			$eventobj->addNode( new ZCiCalDataNode( "DTEND:" . ZCiCal::fromSqlDateTime( $booked_day . " 23:00:00" ) ) );

			// UID is a required item in VEVENT, create unique string for this event
			// Adding your domain to the end is a good way of creating uniqueness
			$uid = date( 'Y-m-d-H-i-s' ) . '-' . $booked_day . "@bfwc";
			$eventobj->addNode( new ZCiCalDataNode( "UID:" . $uid ) );

			// DTSTAMP is a required item in VEVENT
			$eventobj->addNode( new ZCiCalDataNode( "DTSTAMP:" . ZCiCal::fromSqlDateTime() ) );

		}

		return $icalobj->export();

	} else {

		return '';

	}

}

/**
 * Return the path and filename for a calendar
 * @since 2.0.0
 */
function bfwc_get_calendar_path( $product_id ) {

	$dir_path = bfwc_get_calendar_directory_path();

	$calendar_path = trailingslashit( $dir_path ) . 'calendar-' . $product_id . '.ics';
	return $calendar_path;

}

/**
 * Return calendar directoy path
 * Create it if it doesn't exist
 * @since 2.0.0
 */
function bfwc_get_calendar_directory_path() {

	$wp_upload_dir = wp_upload_dir();

	if( ! file_exists( $wp_upload_dir['basedir'] . '/calendars' ) ) {
		wp_mkdir_p( $wp_upload_dir['basedir'] . '/calendars' );
		// Top level blank index.php
		@file_put_contents( $wp_upload_dir['basedir'] . '/calendars' . '/index.html', '' );
	}
	
	$path = $wp_upload_dir['basedir'] . '/calendars';

	return $path;

}

/**
 * Return the URL and filename for a calendar
 */
function bfwc_get_calendar_url( $product_id ) {

	$wp_upload_dir = wp_upload_dir();
	$dir_url = $wp_upload_dir['baseurl'] . '/calendars';
	$url = trailingslashit( $dir_url ) . 'calendar-' . $product_id . '.ics';
	return $url;

}

/**
 * Parse external calendars and block out dates accordingly
 * @param $product_ids	Single product ID or 'all' for all products
 * @param $calendar_id If false, parse all calendars
 * @since 2.0.0
 */
function bfwc_parse_external_calendar( $product_ids='all', $calendar_id=false ) {

	if( $product_ids == 'all' ) {
		$product_ids = bfwc_get_all_bookable_products();
	} else {
		$product_ids = array( $product_ids );
	}

	$start = 1;
	$end = 4; // Total number of calendars

	if( $calendar_id ) {
		// If we've specified a calendar ID, we are only going to sync this one
		$start = $calendar_id;
		$end = $calendar_id;
	}

	foreach( $product_ids as $product_id ) {

		for( $i = $start; $i <= $end; $i++ ) {

			// Use this to store a list of our imported bookings that will need to be blocked
			$imported_bookings = array();
	
			// Get the calendar for this product
			$calendar_url = get_post_meta( $product_id, 'bfwc_calendar_url_' . $i, true );
			$calendar_name = get_post_meta( $product_id, 'bfwc_calendar_name_' . $i, true );

			if( ! $calendar_url ) {
				
				continue;

			} else {

				// Check the file exists
				$headers = get_headers( $calendar_url );
				if( stripos( $headers[0], "200 OK" ) === false || stripos( $headers[0], "200 OK" ) === false ) {
					// continue;
				}
				
			}
	
			try {
	
				// Update this to ensure we're not parsing any dates in the past
				$ical = new ICal( $calendar_url, array(
					'defaultSpan'                 => 2,     // Default value
					'defaultTimeZone'             => 'UTC',
					'defaultWeekStart'            => 'MO',  // Default value
					'disableCharacterReplacement' => false, // Default value
					'filterDaysAfter'             => null,  // Default value
					'filterDaysBefore'            => null,  // Default value
					'httpUserAgent'               => null,  // Default value
					'skipRecurrence'              => false, // Default value
				));
				// $ical->initFile('ICal.ics');
				// $ical->initUrl('https://raw.githubusercontent.com/u01jmg3/ics-parser/master/examples/ICal.ics', $username = null, $password = null, $userAgent = null);
	
			} catch (\Exception $e) {
				die($e);
			}
	
			$events = $ical->eventsFromInterval( apply_filters( 'bfwc_events_from_interval', '1 year' ) );
	
			foreach( $events as $event ) {
	
				$dtstart = $ical->iCalDateToDateTime($event->dtstart_array[3]);
				$dtend = $ical->iCalDateToDateTime($event->dtend_array[3]);
	
				// Ignore last date in a booking?
				$ignore = bfwc_get_exclude_last_day();
	
				// Iterate through the event range and block off dates in booking calendar
				$start_date_ymd = $dtstart->format( 'Y-m-d' );
				$end_date_ymd = $dtend->format( 'Y-m-d' );
	
				/**
				* SKIP ANY DATES THAT FINISH BEFORE TODAY - NO POINT
				*/
	
				if( $ignore ) {
					$end_date = new DateTime( $end_date_ymd );
					$end_date->modify( '-1 day' );
					$end_date_ymd = $end_date->format( 'Y-m-d' );
				}
	
				// Iterate through each date from start to end
				// Use $check_date then add one day
				$check_date = $start_date_ymd;
	
				do {
	
					$check_date = new DateTime( $check_date );
					$date = $check_date->format( 'Y-m-d' );
	
					// Build the array
					$imported_bookings[] = $date;
	
					$check_date->modify( '+1 day' );
					$check_date = $check_date->format( 'Y-m-d' );
	
				} while( $check_date <= $end_date_ymd );
	
			}
	
			// Now look for a list of blocked bookings previously imported for this service
			$already_blocked = get_post_meta( $product_id, 'bfwc_already_blocked_' . $i, true );
			if( ! $already_blocked ) $already_blocked = array();
	
			// Remove any duplicate values in
	
	
			// We need to compare these imported bookings and already booked arrays to find any new dates that need to be blocked
			// This array should include any dates in $imported_bookings that don't appear in $already_blocked
			$need_blocking = array_diff( $imported_bookings, $already_blocked );
	
			if( $need_blocking ) {
	
				foreach( $need_blocking as $date ) {
	
					$used_bookings = bfwc_get_available_bookings( $date, $product_id );
	
					// Products with external calendars can only have a max bookings of 1 - can't have multiple bookings
					if( ! $used_bookings ) {
	
						$booking_meta = array(
							'start_date'		=> $date,
							'end_date'			=> $date
						);
	
						// Add a booking here - give it a status Airbnb (or similar)
						$blocked_meta = array(
							$product_id => array(
								'blocked_by'	=> $calendar_name
							)
						);
	
						// Block out this date
						bfwc_update_booked_days( $product_id, false, $booking_meta, 'std', $blocked_meta );
	
					} else {
	
						// This date could already be booked - add a notice
	
					}
	
				}
	
			}
	
			// Unblock any dates that are no longer booked
			// This array should include any dates that appear in $already_blocked but don't appear in $imported_bookings
			$need_unblocking = array_diff( $already_blocked, $imported_bookings );
	
			if( $need_unblocking ) {
	
				foreach( $need_unblocking as $date ) {
	
					$booking_meta = array(
						'start_date'		=> $date,
						'end_date'			=> $date
					);
	
					// Restore this date
					bfwc_update_booked_days( $product_id, false, $booking_meta, 'restore', false );
	
				}
	
			}
	
			// Finally, save all currently blocked dates for this service
			update_post_meta( $product_id, 'bfwc_already_blocked_' . $i, $imported_bookings );
	
		}


	}

}

/**
 * Get whether to ignore the last day of an imported booking
 * @since 2.0.0
 */
function bfwc_get_exclude_last_day() {

	$exclude = get_option( 'bfwc_exclude_last_day', 'yes' );
	return $exclude;

}

function bfwc_sync_external_calendar() {	

	if( ! isset( $_GET['sync_cal'] ) || ! isset( $_GET['product_id'] ) ) {
		return;
	}

	$calendar_id = ! empty( $_GET['calendar_id'] ) ? $_GET['calendar_id'] : false;

	bfwc_parse_external_calendar( $_GET['product_id'], $calendar_id );

}
add_action( 'init', 'bfwc_sync_external_calendar' );

/**
 * Automatically sync all external calendars at specified interval
 * This is fired when Bookings settings are saved
 * @since 2.0.0
 */
function bfwc_auto_sync_external_calendars() {	

	// Check option is enabled
	if( get_option( 'bfwc_auto_sync', 'no' ) == 'no' ) {
		// Cancel any existing scheduled job
		wp_clear_scheduled_hook( 'bfwc_auto_sync' );
		update_option( 'bfwc_previous_sync_interval', 0 );
		return;
	}

	// What happens if interval is changed?
	$previous_interval = get_option( 'bfwc_previous_sync_interval', false );
	$current_interval = get_option( 'bfwc_sync_interval', 60 );

	if( $previous_interval != $current_interval ) {
		// Update the interval
		bfwc_update_auto_sync_interval( $current_interval );
	}

}

/**
 * Update the auto sync to a new interval
 * @since 2.0.0
 */
function bfwc_update_auto_sync_interval( $current_interval ) {

	// Cancel the existing scheduled job
	wp_clear_scheduled_hook( 'bfwc_auto_sync' );

	// Create a job at the new interval
	if( ! wp_next_scheduled ( 'bfwc_parse_external_calendar' ) ) {
    	wp_schedule_event( time(), 'bfwc_sync', 'bfwc_auto_sync' );
    }

	update_option( 'bfwc_previous_sync_interval', $current_interval );

	// Record next/last sync?

}
add_action( 'bfwc_auto_sync', 'bfwc_parse_external_calendar', 10, 2 );

/**
 * Set a custom interval for syncing
 * @since 2.0.0
 */
function bfwc_cron_schedules( $schedules ) {

	// Check option is enabled
	if( get_option( 'bfwc_auto_sync', 'no' ) == 'no' ) {
		return $schedules;
	}

	$current_interval = get_option( 'bfwc_sync_interval', 60 );
	$schedules['bfwc_sync'] = array(
		'interval' 	=> $current_interval * 60,
		'display'	=> $current_interval . ' ' . __( 'minutes', 'bfwc' )
	);

	return $schedules;

}
add_filter( 'cron_schedules', 'bfwc_cron_schedules' );