<?php
/**
 * Functions for handling Booked Days features
 * @since 1.4.0
 * @package Bookings for WooCommerce
 */

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Make days unavailable
 * @param $booking_id
 * @since 2.0.0
 */
function bfwc_remove_booked_days_by_booking_id( $booking_id ) {

	$start_date = get_post_meta( $booking_id, 'bfwc_start_date', true ); // 2.0.9 changed from bfwc_start_date_text (locale-dependent) to bfwc_start_date (yyyy-mm-dd)
	if( empty( $start_date ) ) {
		return;
	}

	$end_date = get_post_meta( $booking_id, 'bfwc_end_date', true ); // 2.0.9 changed from bfwc_end_date_text (locale-dependent) to bfwc_end_date (yyyy-mm-dd)
	if( empty( $end_date ) ) {
		return;
	}

	$quantity = get_post_meta( $booking_id, 'bfwc_num_bookings', true ) ? get_post_meta( $booking_id, 'bfwc_num_bookings', true ) : 1;
	if( ! $quantity ) {
		return;
	}

	// Remove availability for original dates
	$product_id = get_post_meta( $booking_id, 'bfwc_product_id', true ) ? get_post_meta( $booking_id, 'bfwc_product_id', true ) : false;
	$booking_meta['start_date'] = $start_date;
	$booking_meta['end_date'] = $end_date;
	$booking_meta['num_bookings'] = $quantity;
	bfwc_update_booked_days( $product_id, $booking_id, $booking_meta, 'std' );

}

/**
 * Restore booked days for a specific booking
 * @param $booking_id
 * @since 2.0.0
 */
function bfwc_restore_booked_days_by_booking_id( $booking_id ) {

	$start_date = get_post_meta( $booking_id, 'bfwc_start_date', true ); // 2.0.9 changed from bfwc_start_date_text (locale-dependent) to bfwc_start_date (yyyy-mm-dd)
	if( empty( $start_date ) ) {
		// Add notice @todo
		return;
	}

	$end_date = get_post_meta( $booking_id, 'bfwc_end_date', true ); // 2.0.9 changed from bfwc_end_date_text (locale-dependent) to bfwc_end_date (yyyy-mm-dd)
	if( empty( $end_date ) ) {
		// Add notice @todo
		return;
	}

	$quantity = get_post_meta( $booking_id, 'bfwc_num_bookings', true ) ? get_post_meta( $booking_id, 'bfwc_num_bookings', true ) : 1;
	if( ! $quantity ) {
		// Add notice @todo
		return;
	}

	// Restore availability for original dates
	$product_id = get_post_meta( $booking_id, 'bfwc_product_id', true ) ? get_post_meta( $booking_id, 'bfwc_product_id', true ) : false;
	$booking_meta['start_date'] = $start_date;
	$booking_meta['end_date'] = $end_date;
	$booking_meta['num_bookings'] = $quantity;
	bfwc_update_booked_days( $product_id, $booking_id, $booking_meta, 'restore' );

}

/**
 * Update data for each booked day
 * @param $action std - remove availability | reset - reset original availability | restore - restore cancelled dates
 * @since 1.4.0
 * @param $blocked_meta - additional meta for days blocked by external calendars
 * @since 2.0.0
 */
function bfwc_update_booked_days( $product_id, $booking_id, $booking_meta, $action='std', $blocked_meta=false ) {

	// Need to iterate through each day of the booking
	$start_date = bfwc_date_time( $booking_meta['start_date'] );
	$start_date_ymd = $start_date->format( 'Y-m-d' );
	$end_date = bfwc_date_time( $booking_meta['end_date'] );
	$end_date_ymd = $end_date->format( 'Y-m-d' );

	// Iterate through each date from start to end
	// Use $check_date then add one day
	$check_date = new DateTime( $start_date_ymd );
	$check_date = $check_date->format( 'Y-m-d' );

	// How many bookings we're making per unit
	$num_bookings = ! empty( $booking_meta['num_bookings'] ) ? $booking_meta['num_bookings'] : 1;
	$max_bookings = bfwc_get_max_bookings( $product_id );

	/**
	 * DON'T UPDATE bookings FOR END DATE IF OVERLAP / WEEKLY?
	 * OR SET AN EXTRA PARAM TO INDICATE IT'S A PART UNIT?
	 */

	$is_overlappable = bfwc_is_overlappable( $product_id );

	do {

		$check_date = new DateTime( $check_date );
		$date = $check_date->format( 'Y-m-d' );

		// Has this date already been saved?
		$booked_day = bfwc_get_page_by_title( $date, 'bfwc_booked_days' );

		// If not, create it
		if( ! $booked_day ) {
			$booked_day_id = wp_insert_post( array(
				'post_title'	=> $date,
				'post_type'   	=> 'bfwc_booked_days',
				'post_status'	=> 'publish'
			) );
		} else {
      		$booked_day_id = $booked_day->ID;
    	}

		// If the date is the last data of the booking and it's overlappable, then don't add 1 to used_bookings
		if( $date == $end_date_ymd && $is_overlappable == 'yes' ) {
			$add_booking = 0;
		} else {
			$add_booking = $num_bookings;
		}

		// Check meta
		$booked_meta = get_post_meta( $booked_day_id, 'booking_meta', true );

		if( ! $booked_meta ) {

			/**
			 * Save new data
			 * $booked_meta params
			 * @param used_bookings	total bookings already made for this product on this date
			 * @param booking_ids array of IDs for bookings for this product on this date
			 * @param used_by_booking array of units booked per booking ID for this product on this date
			 */

			$booked_meta = array(
				$product_id => array(
					'used_bookings'	 	=> $add_booking,
					'booking_ids'	   	=> array( $booking_id ),
					'used_by_booking'	=> array(
						$booking_id => $add_booking
					)
				)
			);

		} else {

			if( $action == 'reset' ) {

				// We're resetting the availability so remove this entry
				unset( $booked_meta[$product_id] );

			} else if( $action == 'restore' ) {

				// We're restoring previously unavailable dates
				// If an order has been cancelled, for instance
				// Or if a booking is removed from the cart (since 1.4.7)

				// Check if this order ID has already been saved
				$booking_ids = isset( $booked_meta[$product_id]['booking_ids'] ) ? $booked_meta[$product_id]['booking_ids'] : array();
				$used_by_booking = isset( $booked_meta[$product_id]['used_by_booking'] ) ? $booked_meta[$product_id]['used_by_booking'] : array();

				// Make num_bookings negative so that it is added back to availability
				//$used_bookings = $booked_meta[$product_id]['used_bookings'] - $num_bookings; // commented out in 1.4.7
				$used_bookings = ! empty( $booked_meta[$product_id]['used_bookings'] ) ? $booked_meta[$product_id]['used_bookings'] : false; // since 1.4.7
				// Remove the booking ID
				//unset( $booked_meta[$product_id]['booking_ids'][$booking_id] ); // booking_ids are saved as index => booking_id, so this does not work anymore? commented out in 1.4.7

				// since 1.4.7
				if ( ! empty( $booking_ids ) ) {
					$tmp = array();
					foreach ( $booking_ids as $bid ) {
						if ( $booking_id != $bid ) {
							// we are not removing this booking id, so save it
							$tmp[] = $bid;
						} else {
							// remove this booking from used_by_booking as well
							unset( $used_by_booking[$booking_id] );
							// subtract
							$used_bookings -= $add_booking; // $add_booking is almost the same as $num_bookings, but $add_booking is 0 if this is the end date
						}
					}
					$booking_ids = $tmp; // update
				}

				$booked_meta[$product_id] = array(
					'used_bookings'	 	=> $used_bookings,
					'booking_ids'		=> $booking_ids,
					'used_by_booking'	=> $used_by_booking
				);

			} else {

				// std action

				if( ! isset( $booked_meta[$product_id] ) ) {

					// Create data

					// If an entry for this product ID does not already exist
					$booked_meta[$product_id] = array(
						'used_bookings'	 	=> $add_booking,
						'booking_ids'	   	=> array( $booking_id ),
						'used_by_booking'	=> array(
							$booking_id => $add_booking
						)
					);

				} else {

					// Update existing data

					// Check if this order ID has already been saved
					$booking_ids = isset( $booked_meta[$product_id]['booking_ids'] ) ? $booked_meta[$product_id]['booking_ids'] : array();

			        if( ! in_array( $booking_id, $booking_ids ) ) {

						// Add booking
	          			$used_bookings = $booked_meta[$product_id]['used_bookings'] + $add_booking;
						// The used number of bookings can never be more than the max
						$used_bookings = min( $max_bookings, $used_bookings );
						if ( ! empty ( $booked_meta[$product_id]['used_by_booking'][$booking_id] ) ) {
							$booked_meta[$product_id]['used_by_booking'][$booking_id] += $add_booking;
						} else {
							$booked_meta[$product_id]['used_by_booking'][$booking_id] = $add_booking;
						}
		  				$booked_meta[$product_id]['booking_ids'][] = $booking_id;
		  				$booked_meta[$product_id] = array(
		  					'used_bookings'		=> $used_bookings,
							'booking_ids'	  	=> $booked_meta[$product_id]['booking_ids'],
		  					'used_by_booking'	=> $booked_meta[$product_id]['used_by_booking']
		  				);

			        } else if( in_array( $booking_id, $booking_ids ) ) {

						// Update existing booking

						//$used_bookings = $booked_meta[$product_id]['used_bookings'] + $add_booking;
						//$used_bookings = min( $max_bookings, $used_bookings );
						//if ( ! empty ( $booked_meta[$product_id]['used_by_booking'][$booking_id] ) ) {
						//	$booked_meta[$product_id]['used_by_booking'][$booking_id] += $add_booking;
						//} else {
						//	$booked_meta[$product_id]['used_by_booking'][$booking_id] = $add_booking;
						//}

						// 2.0.4, let's not add the count, but update it, perhaps the num_bookings was decreased for the booking
						$booked_meta[$product_id]['used_by_booking'][$booking_id] = $add_booking;
						// then let's recount used_by_booking so that we have an accurate count of used bookings
						$used_bookings = 0;
						foreach ( $booked_meta[$product_id]['used_by_booking'] as $booking_id => $booking_count ) {
							$used_bookings += $booking_count;
						}
						// The used number of bookings can never be more than the max
						$used_bookings = min( $max_bookings, $used_bookings );

						//$booked_meta[$product_id] = array(
		  				//	'used_bookings'	 => $used_bookings
		  				//);

						// 2.0.4, only update used_bookings so that we don't lose other data
						$booked_meta[$product_id]['used_bookings'] = $used_bookings;

					} else if( $booking_id === 0 ) {

						// If booking ID is set to 0, we are updating non std dates from the admin
						// Or blocking a date from an external calendar
						//$booked_meta[$product_id] = array(
		  				//	'used_bookings'	 => $num_bookings
		  				//);

						// 2.0.4, in case this date is already partially blocked, we don't want to lose the other data maybe, so get used_bookings from real bookings
						$used_bookings = 0;
						if ( ! empty( $booked_meta[$product_id]['used_by_booking'] ) ) {
							foreach ( $booked_meta[$product_id]['used_by_booking'] as $booking_id => $booking_count ) {
								$used_bookings += $booking_count;
							}
						}
						// add the num_bookings from non std dates or date from an external calendar
						$used_bookings += $num_bookings;
						// The used number of bookings can never be more than the max
						$used_bookings = min( $max_bookings, $used_bookings );
						$booked_meta[$product_id]['used_bookings'] = $used_bookings;

					}

				}

			}

		}

		// Set the start and end date for the booking so that it's queryable?

		update_post_meta( $booked_day_id, 'booking_meta', $booked_meta );
		update_post_meta( $booked_day_id, 'blocked_meta', $blocked_meta );

		$check_date->modify( '+1 day' );
		$check_date = $check_date->format( 'Y-m-d' );

	} while( $check_date <= $end_date_ymd );

}

/**
 * Check how many unbooked bookings are available for a specific date
 * @param $booking_id if specified, exclude bookings for specified booking ID
 * @since 1.4.0
 */
function bfwc_get_available_bookings( $date, $product_id, $booking_id=false ) {

  $booked_day = bfwc_get_page_by_title( $date, 'bfwc_booked_days' );

  if( $booked_day ) {

    // Get the number of used bookings for this date
    $booked_day_id = $booked_day->ID;
	$booked_meta = get_post_meta( $booked_day_id, 'booking_meta', true );
    $blocked_meta = get_post_meta( $booked_day_id, 'blocked_meta', true );

    if( ! empty( $booked_meta[$product_id]['used_bookings'] ) ) {

		$used_bookings = $booked_meta[$product_id]['used_bookings'];
		// Do we need to adjust to exclude a specific booking ID?
		if( $booking_id && ! empty( $booked_meta[$product_id]['used_by_booking'][$booking_id] ) ) {
			$used_bookings = $used_bookings - $booked_meta[$product_id]['used_by_booking'][$booking_id];
		}

      if( $used_bookings < 0 ) $used_bookings = 0;
      return $used_bookings;

    }

  }

  return 0;

}

/**
 * Check if any dates in a specified range have used all bookings
 * @return Boolean  false if any date in the range has no available bookings
 * @since 1.4.0
 */
function bfwc_check_bookings_in_range( $start_date, $end_date, $product_id, $max_bookings ) {

  	$start_date = new DateTime( $start_date );
	$start_date_ymd = $start_date->format( 'Y-m-d' );
	$end_date = new DateTime( $end_date );
	$end_date_ymd = $end_date->format( 'Y-m-d' );

	// Iterate through each date from start to end
	// Use $check_date then add one day
	$check_date = new DateTime( $start_date_ymd );
	$check_date = $check_date->format( 'Y-m-d' );

	do {

		$check_date = new DateTime( $check_date );
		$date = $check_date->format( 'Y-m-d' );

		// Does this date have any used bookings?
		$used_bookings = bfwc_get_available_bookings( $date, $product_id );
    if( $used_bookings >= $max_bookings ) {
      // There's a date where there are no available bookings
      return false;
    }

		$check_date->modify( '+1 day' );
		$check_date = $check_date->format( 'Y-m-d' );

	} while( $check_date <= $end_date_ymd );

  return true;

}

/**
 * Return an array of available bookings per day
 * This is used in the title attribute of the calendar cells
 * @since 1.4.0
 */
function bfwc_get_availability_per_day( $product_id=false ) {

  	if( ! $product_id ) {
		$product_id = get_the_ID();
	}

	$max_bookings = bfwc_get_max_bookings( $product_id );

	$availability = array();

	$start_date = new DateTime( 'NOW' );

	// Check if there's a last date
	$last_date = bfwc_get_last_bookable_date( $product_id );
	if( $last_date ) {

		// Check if the date is valid (e.g. could be non-English date string)
		if( ! strtotime( $last_date ) ) {
			// Try to get valid date
			$last_date = bfwc_get_last_bookable_date_ymd( $product_id );
		}
		$end_date = new DateTime( $last_date );

	} else {

		// Find the last booked date
		// Anything after that is fully available
		$last_booked_day = bfwc_get_last_booked_date_by_product( $product_id );
		if( $last_booked_day ) {
			$end_date = new DateTime( $last_booked_day );
		} else {
			$end_date = new DateTime( 'NOW' );
		}

	}

	$start_date_ymd = $start_date->format( 'Y-m-d' );
	$end_date_ymd = $end_date->format( 'Y-m-d' );

	// Iterate through each date from start to end
	// Use $check_date then add one day
	$check_date = new DateTime( $start_date_ymd );
	$check_date = $check_date->format( 'Y-m-d' );

	do {

		$check_date = new DateTime( $check_date );
		$date = $check_date->format( 'Y-m-d' );

		// Does this date have any used bookings?
		$used_bookings = bfwc_get_available_bookings( $date, $product_id );
		
		// Store the available bookings
		$availability[$date] = $max_bookings - $used_bookings;

		$check_date->modify( '+1 day' );
		$check_date = $check_date->format( 'Y-m-d' );

	} while( $check_date <= $end_date_ymd );

	return $availability;

}

/**
 * Get the availability per day for a newly created booking
 * @since 2.0.0
 */
function bfwc_get_availability_per_day_new_booking( ) {

	// Check nonce and permissions
	if( ! isset( $_POST['security'] ) || ! wp_verify_nonce( $_POST['security'], 'bfwc_add_product_new_booking' ) ) {
		wp_send_json( array( 'result' => 'nonce fail' ) );
		exit;
	}

	$product_id = $_REQUEST['product_id'] ? $_REQUEST['product_id'] : false;
	$availability = bfwc_get_availability_per_day( $product_id );
	wp_send_json_success(
		array(
			'availability' 	=> $availability,
			'max_bookings'	=> bfwc_get_max_bookings( $product_id )
		)
	);

}
add_action( 'wp_ajax_bfwc_get_availability_per_day_new_booking', 'bfwc_get_availability_per_day_new_booking' );

/**
 * Find the last date that a product is booked out for
 * @since 1.4.0
 */
function bfwc_get_last_booked_date_by_product( $product_id ) {

  	// Do existing bookings last to ensure they don't get removed
	$args = array(
		'post_type'			=> 'bfwc_booked_days',
		'post_status'		=> 'publish',
		'posts_per_page'	=> 1,
		'orderby'         	=> 'title',
		'order'           	=> 'DESC'
	);

	// Just get booking meta for this product ID
	$args['meta_query'] = array(
		'relation'		=> 'AND',
		array(
			'key'		=> 'booking_meta',
			'value'		=> $product_id,
			'compare'	=> 'LIKE'
		)
	);

  	$last_booked_day = false;

	$booked_days = new WP_Query( $args );
	if( $booked_days->have_posts() ) {
    // The first entry should be the last booked date
		foreach( $booked_days->posts as $booked_day ) {
      		$last_booked_day = $booked_day->post_title;
    	}
  	}

  	return $last_booked_day;

}

/**
 * Iterates through a range of dates to find the lowest available number of bookings
 * This is used on the front end to set the max attribute of the num_bookings input field
 * @param $booking_id if specified, discount bookings already assigned to specific booking id
 * @since 1.4.0
 */
function bfwc_get_maximum_booking_by_range( $product_id, $start_date_ymd, $end_date_ymd, $booking_id=false ) {

  $start_date = new DateTime( $start_date_ymd );
  $end_date = new DateTime( $end_date_ymd );

  $start_date_ymd = $start_date->format( 'Y-m-d' );
  $end_date_ymd = $end_date->format( 'Y-m-d' );

  // Iterate through each date from start to end
  // Use $check_date then add one day
  $check_date = new DateTime( $start_date_ymd );
  $check_date = $check_date->format( 'Y-m-d' );

  // Start with the default max bookings
  $max_bookings = bfwc_get_max_bookings( $product_id );
  $lowest_availability = $max_bookings;

  do {

    $check_date = new DateTime( $check_date );
    $date = $check_date->format( 'Y-m-d' );

    // Does this date have any used bookings?
    $used_bookings = bfwc_get_available_bookings( $date, $product_id, $booking_id );

    // How many bookings are available for this date?
    $availability = $max_bookings - $used_bookings;

    // Set max bookings to the lowest number
    $lowest_availability = min( $availability, $lowest_availability );

    $check_date->modify( '+1 day' );
    $check_date = $check_date->format( 'Y-m-d' );

  } while( $check_date <= $end_date_ymd );

  return $lowest_availability;

}

/**
 * Restore blocked dates when removed from Custom Pricing field
 * @since 1.4.0
 */
function bfwc_restore_availability() {

	// Check nonce and permissions
	if( ! isset( $_POST['security'] ) || ! wp_verify_nonce( $_POST['security'], 'bfwc_nonstd_nonce' ) ) {
		wp_send_json( array( 'result' => 'nonce fail' ) );
		exit;
	}

	$start_date = $_POST['start_date'] ? $_POST['start_date'] : false;
	$end_date = $_POST['end_date'] ? $_POST['end_date'] : false;
	$product_id = $_POST['product_id'] ? $_POST['product_id'] : false;
	if( ! $start_date || ! $end_date || ! $product_id ) {
		wp_send_json( array( 'result' => 'missing data' ) );
		exit;
	}

	$booking_meta['start_date'] = $start_date;
	$booking_meta['end_date'] = $end_date;
	$booking_meta['num_bookings'] = 0;
	//bfwc_update_booked_days( $product_id, 0, $booking_meta, true ); // setting $action to true satisfies the condition $action == 'reset', could cause issues
	bfwc_update_booked_days( $product_id, 0, $booking_meta, 'restore' ); // 2.0.4

	wp_send_json( array( 'done' ) );

}
add_action( 'wp_ajax_bfwc_restore_availability', 'bfwc_restore_availability' );

/**
 * Get all booked days by product ID
 * @since 2.0.0
 */
function bfwc_get_bookings_by_id( $product_id ) {

  	// Get all booked days
	$args = array(
		'post_type'			=> 'bfwc_booked_days',
		'post_status'		=> 'publish',
		'posts_per_page'	=> -1,
		'orderby'         	=> 'title',
		'order'           	=> 'DESC'
	);

	// Just get booking meta for this product ID
	$args['meta_query'] = array(
		'relation'		=> 'AND',
		array(
			'key'			=> 'booking_meta',
			'value'		=> $product_id,
			'compare'	=> 'LIKE'
		)
	);

  	$booked_days = false;

	$bookings = new WP_Query( $args );
	if( $bookings->have_posts() ) {
    // The first entry should be the last booked date
	foreach( $bookings->posts as $booked_day ) {
    	$booked_days[] = $booked_day->post_title;
    }
  }

  return $booked_days;

}

/**
 * Get all days blocked by external services, e.g. Airbub, by product ID
 * @return Array
 * @since 2.0.0
 */
function bfwc_get_blocked_bookings_by_id( $product_id ) {

  // Get all booked days
	$args = array(
		'post_type'				=> 'bfwc_booked_days',
		'post_status'			=> 'publish',
		'posts_per_page'	=> -1,
    'orderby'         => 'title',
    'order'           => 'DESC'
	);

	// Just get booking meta for this product ID which have been blocked by an external service
	$args['meta_query'] = array(
		'relation'		=> 'AND',
		array(
			'key'			=> 'booking_meta',
			'value'		=> $product_id,
			'compare'	=> 'LIKE'
		),
		array(
			'key'			=> 'blocked_meta',
			'value'		=> $product_id,
			'compare'	=> 'LIKE'
		)
	);

  $blocked_days = array();

	$blocked_query = new WP_Query( $args );
	if( $blocked_query->have_posts() ) {
    // The first entry should be the last booked date
		foreach( $blocked_query->posts as $blocked_day ) {
      $blocked_days[$blocked_day->post_title] = get_post_meta( $blocked_day->ID, 'blocked_meta', true );
    }
  }

  return $blocked_days;

}

/**
 * Get all dates blocked by external services, e.g. Airbub, by product ID
 * @since 2.0.0
 */
function bfwc_get_blocked_dates_by_id( $product_id ) {

  	$blocked = bfwc_get_blocked_bookings_by_id( $product_id );
	$blocked = array_keys( $blocked );
	return json_encode( $blocked );

}

/**
 * Get all the metadata for a specific date
 * @param $date Date Y-m-d
 * @since 2.0.0
 */
function bfwc_get_booked_day_metadata( $date ) {

	$metadata = array();

	$date = new DateTime( $date );
  	$ymd = $date->format( 'Y-m-d' );

	$booked_day = bfwc_get_page_by_title( $ymd, 'bfwc_booked_days' );

  	if( $booked_day ) {

		$booked_day_id = $booked_day->ID;
		// $metadata['booked_meta'] = get_post_meta( $booked_day_id, 'booking_meta', true );
		$metadata['booking_meta'] = get_post_meta( $booked_day_id, 'booking_meta', true );
		
		if( $metadata['booking_meta'] ) {

			foreach( $metadata['booking_meta'] as $product_id=>$data ) {

				$booking_ids= array();

				if( ! empty( $data['booking_ids']  )) {
					foreach( $data['booking_ids'] as $booking_id ) {
						// Ensure only active bookings are included
						if( get_post_status( $booking_id ) == 'publish' ) {
							$booking_ids[] = $booking_id;
						}
					}

					$metadata['booking_meta'][$product_id]['booking_ids'] = $booking_ids;
				}

			}			

		}
		
		$metadata['blocked_meta'] = get_post_meta( $booked_day_id, 'blocked_meta', true );

	}

	return $metadata;

}

/**
 * Edit the availability of a date
 * @since 2.0.0
 */
function bfwc_edit_availability() {

	// @todo

	// Check nonce and permissions
	// if( ! isset( $_POST['security'] ) || ! wp_verify_nonce( $_POST['security'], 'bfwc_nonstd_nonce' ) ) {
	// 	wp_send_json( array( 'result' => 'nonce fail' ) );
	// 	exit;
	// }

	$date = $_POST['date'] ? $_POST['date'] : false;
	$product_id = $_POST['product_id'] ? $_POST['product_id'] : false;
	if( ! $date || ! $product_id ) {
		wp_send_json( array( 'result' => 'missing data' ) );
		exit;
	}

	$metadata = bfwc_get_booked_day_metadata( $date );

	// $booking_meta['start_date'] = $start_date;
	// $booking_meta['end_date'] = $end_date;
	// $booking_meta['num_bookings'] = 0;
	// bfwc_update_booked_days( $product_id, 0, $booking_meta, true );

	wp_send_json( array( 'done' ) );

}
add_action( 'wp_ajax_bfwc_edit_availability', 'bfwc_edit_availability' );
