<?php
/**
 * Functions for setting up the Booking product type
 * @since 1.0.0
 * @package Bookings for WooCommerce
 */

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Enqueue some scripts
 * bookings.js only enqueued on product page
 */
function bfwc_enqueue_scripts() {

	$post_id = get_the_ID();

	$date_format = get_option( 'date_format' );
	$date_format = bfwc_get_date_format_for_js( $date_format );

	$lang = bfwc_get_language_from_locale();

	if( 'product' === get_post_type( $post_id ) ) {

		$product = wc_get_product( $post_id );

		if( 'simple_booking' === $product->get_type() ) {

			wp_enqueue_script( 'bfwc-script', trailingslashit( BFWC_PLUGIN_URL ) . 'assets/js/bookings.js', array( 'jquery', 'jquery-ui-datepicker' ), BFWC_PLUGIN_VERSION, true );

			$label = bfwc_get_unit_label( get_the_ID() );
			$postpone_start = apply_filters( 'bfwc_postpone_start', get_post_meta( $post_id, '_postpone_start', true ), $post_id );
			if( ! $postpone_start ) $postpone_start = 0;

			wp_localize_script(
				'bfwc-script',
				'bfwc_params',
				array(
					'ajaxurl' 					=> admin_url( 'admin-ajax.php' ),
					'bookings_start'			=> get_post_meta( $post_id, '_bookings_start', true ),
					'minimum'					=> bfwc_get_minimum_duration( $post_id ),
					'last_date'					=> bfwc_get_last_bookable_date( $post_id ),
					'last_date_ymd'				=> bfwc_get_last_bookable_date_ymd( $post_id ),
					'allow_overlaps'			=> bfwc_get_allow_overlaps( $post_id ),
					'booking_unit'				=> bfwc_get_booking_unit( $post_id ),
					'bookings_unavailable'		=> bfwc_get_unavailable_dates( $post_id ),
					'date_format'				=> $date_format,
					'label_single'				=> isset( $label['single'] ) ? strtolower( $label['single'] ) : false,
					'label_plural'				=> isset( $label['plural'] ) ? strtolower( $label['plural'] ) : false,
					'currency_symbol'			=> get_woocommerce_currency_symbol(),
					'buffer'					=> get_post_meta( $post_id, '_buffer', true ),
					'postpone_start'			=> $postpone_start,
					'discount_text'				=> __( 'discount', 'bfwc' ),
					'availability_text'			=> __( 'available', 'bfwc' ),
					'max_bookings'				=> bfwc_get_max_bookings( $post_id ),
					'language'					=> apply_filters( 'bfwc_language', $lang ),
					'first_day'					=> get_option( 'bfwc_first_day', '0' ),
					'wc_decimals'				=> absint( apply_filters( 'wc_get_price_decimals', get_option( 'woocommerce_price_num_decimals', 2 ) ) ),
					'prices'					=> bfwc_get_prices_per_day(),
					'std_cost'					=> bfwc_get_cost( $product ),
					'auto_set_end'				=> bfwc_get_auto_set_end( $post_id )
				)
			);

			wp_add_inline_script(
				'bfwc-script',
				'var bfwc_availability = ' . json_encode( bfwc_get_availability_per_day() ),
				'before'
			);
		}

		wp_enqueue_script( 'bfwc-search-script', trailingslashit( BFWC_PLUGIN_URL ) . 'assets/js/bookings-search.js', array( 'jquery', 'jquery-ui-datepicker' ), BFWC_PLUGIN_VERSION, true );

		// If we've got a localised file, load it
		$file_url = trailingslashit( BFWC_PLUGIN_URL ) . 'assets/js/i18n/datepicker-' . $lang . '.js';
		$file_path = trailingslashit( BFWC_PLUGIN_DIR_PATH ) . 'assets/js/i18n/datepicker-' . $lang . '.js';
		if( file_exists( $file_path ) ) {
			wp_enqueue_script( 'bfwc-datepicker-' . $lang, $file_url, array( 'jquery-ui-datepicker' ), BFWC_PLUGIN_VERSION, true );
		}

		wp_localize_script(
			'bfwc-search-script',
			'bfwc_search_params',
			array(
				'ajaxurl' 							=> admin_url( 'admin-ajax.php' ),
				'date_format'						=> $date_format,
				'language'							=> apply_filters( 'bfwc_language', $lang ),
				'first_day'							=> get_option( 'bfwc_first_day', '0' ),
			)
		);

	}

	wp_enqueue_style( 'bfwc-style', trailingslashit( BFWC_PLUGIN_URL ) . 'assets/css/style.css', array(), BFWC_PLUGIN_VERSION );
	wp_enqueue_style( 'dashicons' );

}
add_action( 'wp_enqueue_scripts', 'bfwc_enqueue_scripts' );

/**
 * Get a two-letter language code from the site locale
 * @since 1.4.6
 */
function bfwc_get_language_from_locale() {
	$locale = get_locale();
	$lang = substr( get_locale(), 0, 2 );
	return $lang;
}

/**
 * Filter body class
 */
function bfwc_filter_body_classes( $classes ) {
	if( bfwc_is_simple_booking_product() ) {
		$view = 'double';
		$classes[] = 'bfwc-view-' . esc_attr( $view );
	}
	if( bfwc_get_optimise_colours() == 'yes' ) {
		$classes[] = 'bfwc-optimise-colours';
	}
	return $classes;
}
add_filter( 'body_class', 'bfwc_filter_body_classes' );

/**
 * Format a date according to site settings
 * @since 2.0.0
 */
function bfwc_format_date( $date ) {

	$date_format = get_option( 'date_format' ); // no longer in use since 1.4.4
	$formatted_date = date( $date_format, strtotime( $date ) );

	return $formatted_date;

}

/**
 * Get equivalent date format for JavaScript
 */
function bfwc_get_date_format_for_js( $date_format ) {
	switch( $date_format ) {
    // WP date formats
		case 'j F Y':
        return( 'dd MM yy' );
        break;
		case 'j, F Y':
        return( 'dd, MM yy' );
        break;
		case 'j. F Y':
        return( 'dd. MM yy' );
        break;
		case 'F j, Y':
        return( 'MM dd, yy' );
        break;
    case 'Y/m/d':
        return( 'yy/mm/dd' );
        break;
    case 'm/d/Y':
        return( 'mm/dd/yy' );
        break;
    case 'd/m/Y':
        return( 'dd/mm/yy' );
        break;
  }
	// Fallback
	return( 'MM dd, yy' );
}

/**
 * Get the standard cost for a product
 * @param $product Mixed int | object
 */
function bfwc_get_standard_cost( $product ) {
	if( is_object( $product ) ) {
		$id = $product->get_id();
	} else {
		$id = $product;
	}
	$std_cost = get_post_meta( $id, '_std_booking_cost', true );
	return apply_filters( 'bfwc_standard_price', $std_cost, $id );
}

/**
 * Filter the product price for role-based prices
 * @since 1.4.8
 */
function bfwc_get_role_price( $price, $product_id ) {

	$roles = bfwc_get_current_user_roles();
	if( $roles ) {
		foreach( $roles as $role ) {
			$role_price = get_post_meta( $product_id, '_std_booking_cost_' . $role, true );
			if( $role_price ) return $role_price;
		}
	}

	return $price;

}
add_filter( 'bfwc_standard_price', 'bfwc_get_role_price', 10, 2 );

/**
 * Get a list of all current user roles
 * @return Array
 */
function bfwc_get_current_user_roles() {

	if( ! is_user_logged_in() ) {
		// Not logged in so can't have a role
		return array();
	}

	$user = wp_get_current_user();
	$roles = ( array ) $user->roles;
	return $roles;

}

/**
 * Get the cost for a product, including tax if specified
 * @param $product Mixed int | object
 */
function bfwc_get_cost( $product ) {

	if( is_object( $product ) ) {
		$id = $product->get_id();
	} else {
		$id = $product;
		$product = wc_get_product( $id );
	}
	if( ! is_object( $product ) ) return false;

	$std_cost = get_post_meta( $id, '_std_booking_cost', true );
	if ( 'incl' === get_option( 'woocommerce_tax_display_shop' ) ) {
		$price = wc_get_price_including_tax( $product, array( 'price' => $std_cost ) );
	} else {
		$price = wc_get_price_excluding_tax( $product, array( 'price' => $std_cost ) );
	}

	$price = html_entity_decode( strip_tags( wc_price( $price) ) );
	return $price;
	
}

/**
 * Get the last bookable date for a product
 */
function bfwc_get_last_bookable_date( $product_id ) {
	$last_date = get_post_meta( $product_id, '_last_date', true );
	return $last_date;
}

/**
 * Get the last bookable date for a product in the yyyy-mm-dd format
 */
function bfwc_get_last_bookable_date_ymd( $product_id ) {
	$last_date = get_post_meta( $product_id, '_last_date_ymd', true );
	// Assign an arbitrary end date
	if( ! $last_date ) $last_date = '2030-12-31';
	return $last_date;
}

/**
 * Get unavailable dates
 * We pass this to the JS to determine which dates should be disabled
 */
function bfwc_get_unavailable_dates( $product_id ) {

	$unavailable_dates = '';
	$bookings_array = array();
	$gather_dates = array();

	$duration = bfwc_get_booking_unit_duration( $product_id );
	$unit = bfwc_get_booking_unit( $product_id );
	$minimum = bfwc_get_minimum_duration( $product_id );
	$maximum = bfwc_get_maximum_duration( $product_id );
	$multiple = absint( $duration ) * absint( $minimum );

	// The number of bookings that can be booked
	$max_bookings = bfwc_get_max_bookings( $product_id );

	// Get any dates that have been manually blocked out
	$nonstd_data = get_post_meta( $product_id, '_nonstd_data', true );

	// Flip the order of dates
	// The user prioritises dates by placing them higher in the list
	// We work through the list backwards
	if( $nonstd_data ) {

		$nonstd_data = array_reverse( $nonstd_data );

		foreach( $nonstd_data as $key=>$nonstd_item ) {
			// Needs all three parameters to be set
			// If the cost is set to 0 then add to the list of unavailable dates
			// If the cost is not set to 0, then remove from the list of unavailable dates
			if( isset( $nonstd_item['start_date'] ) && isset( $nonstd_item['end_date'] ) && isset( $nonstd_item['cost'] ) && $nonstd_item['cost'] == 0 ) {
				if( isset( $nonstd_item['start_date_ymd'] ) ) {
					// Added start_date_ymd in 1.3.0 to prevent foreign language dates throwing an error
					$start_date = $nonstd_item['start_date_ymd'];
				} else {
					// Fallback to dates entered pre-1.3.0
					$start_date = $nonstd_item['start_date'];
				}
				if( isset( $nonstd_item['end_date_ymd'] ) ) {
					// Added end_date_ymd in 1.3.0 to prevent foreign language dates throwing an error
					$end_date = $nonstd_item['end_date_ymd'];
				} else {
					// Fallback to dates entered pre-1.3.0
					$end_date = $nonstd_item['end_date'];
				}
				$buffer = get_post_meta( $product_id, '_buffer', true );

				$new_dates = bfwc_get_simple_dates_array( $start_date, $end_date, $buffer );
				$gather_dates = array_merge( $gather_dates, $new_dates );

			} else if( isset( $nonstd_item['start_date'] ) && isset( $nonstd_item['end_date'] ) && isset( $nonstd_item['cost'] ) && $nonstd_item['cost'] > 0 ) {
				// Remove dates from array (if they are a higher priority)
				if( isset( $nonstd_item['start_date_ymd'] ) ) {
					// Added start_date_ymd in 1.3.0 to prevent foreign language dates throwing an error
					$start_date = $nonstd_item['start_date_ymd'];
				} else {
					// Fallback to dates entered pre-1.3.0
					$start_date = $nonstd_item['start_date'];
				}
				if( isset( $nonstd_item['end_date_ymd'] ) ) {
					// Added end_date_ymd in 1.3.0 to prevent foreign language dates throwing an error
					$end_date = $nonstd_item['end_date_ymd'];
				} else {
					// Fallback to dates entered pre-1.3.0
					$end_date = $nonstd_item['end_date'];
				}
				$buffer = get_post_meta( $product_id, '_buffer', true );
				$remove_dates = bfwc_get_simple_dates_array( $start_date, $end_date, $buffer );

				if( ! empty( $gather_dates ) && ! empty( $remove_dates ) ) {
					$gather_dates = array_diff( $gather_dates, $remove_dates );
				}
			}
		}
	}

	// Do existing bookings last to ensure they don't get removed
	$args = array(
		'post_type'			=> 'bfwc_booking',
		'post_status'		=> 'publish',
		'posts_per_page'	=> -1,
		// 'meta_key'			=> 'bfwc_start_date',
		'fields'			=> 'ids' // Just return the IDs
	);

	// All bookings from today onwards
	$args['meta_query'] = array(
		'relation'		=> 'AND',
		// array(
		// 	'key'		=> 'bfwc_start_date',
		// 	'value'		=> date( 'Y-m-d' ),
		// 	'compare'	=> '>=',
		// 	'type'		=> 'DATE'
		// ),
		array(
			'key'		=> 'bfwc_status',
			'value'		=> 'removed',
			'compare'	=> '!='
		),
		array(
			'key'		=> 'bfwc_product_id',
			'value'		=> $product_id,
			'compare'	=> '='
		)
	);

	// For non-weekly bookings, only get dates from today onwards
	// This is to ensure days in the middle of weekly bookings aren't available
	if( $unit != 'week' ) {
		$args['meta_query'][] = array(
			'key'		=> 'bfwc_start_date',
			'value'		=> date( 'Y-m-d' ),
			'compare'	=> '>=',
			'type'		=> 'DATE'
		);
	}

	$bookings = new WP_Query( $args );
	if( $bookings->have_posts() ) {
		foreach( $bookings->posts as $booking_id ) {
			// Get the start date and the number of units and calculate all dates forward
			$start_date = get_post_meta( $booking_id, 'bfwc_start_date', true );
			$end_date = get_post_meta( $booking_id, 'bfwc_end_date', true );
			$num_units = get_post_meta( $booking_id, 'bfwc_num_units', true );
			$buffer = get_post_meta( $booking_id, '_buffer', true );
			if( ! $buffer ) $buffer = 0;

			/**
			 * Weekly bookings wrap the buffer inside the booking duration
			 * Daily and nightly bookings use the buffer to push out a period of unavailability after the booking finishes
			 */
			if( $unit == 'week' ) {
				$buffer = max( 1, $buffer ); // Ensures the last date of a previous booking is available as the start date of the next booking
				$end_date_obj = new DateTime( $end_date );
				// If this is a weekly booking, count back the number of days in the buffer from the end date
				$end_date = $end_date_obj->modify( '-' . absint( $buffer ) . ' day' );
				$end_date = date( 'Y-m-d', $end_date->getTimestamp() );
			} else if( $unit == 'night' ) {

				// Overnight bookings without a buffer should be bookable from the end date of the previous booking
				$end_date = get_post_meta( $booking_id, 'bfwc_end_date_plus_buffer', true );
				$end_date_obj = new DateTime( $end_date );

				// Check if the booking is overlappable
				$allow_overlaps = bfwc_is_overlappable( $product_id );
				if( $allow_overlaps != 'yes' ) {
					// If the booking is not overlappable
					$end_date = $end_date_obj;
				} else {
					// Pull back one day so that we can start bookings the same day as the previous booking ends
					$end_date = $end_date_obj->modify( '-1 day' );
				}
				$end_date = date( 'Y-m-d', $end_date->getTimestamp() );

			} else {
				$end_date = get_post_meta( $booking_id, 'bfwc_end_date_plus_buffer', true );
			}

			// The end should never be less than the start
			if( $end_date < $start_date ) $end_date = $start_date;

			$new_dates = bfwc_get_simple_dates_array( $start_date, $end_date, $buffer );

			// Check if there are more than one asset available for each of these dates
			if( $max_bookings > 1 ) {

				foreach( $new_dates as $index=>$date ) {

					// Check if this date already has bookings booked
					$used_bookings = bfwc_get_available_bookings( $date, $product_id );

					if( $max_bookings > $used_bookings ) {
						// If the number of allowed bookings is greater than the number of booked bookings, then this date is still available
						unset( $new_dates[$index] );
					}

				}

			}

			$gather_dates = array_merge( $gather_dates, $new_dates );

		}

	}

	wp_reset_query();

	if( ! empty( $gather_dates ) ) {
		sort( $gather_dates );
		$unavailable_dates = implode( ',', $gather_dates );
	}

	// Return an array of all dates that are booked out
	return $unavailable_dates;

}

/**
 * Get an array of dates to use in the calendar
 * @param $start_date	The starting date for the list
 * @param $end_date		The end date for the list
 * @param $buffer			Number of days before end date that booking finishes
 * @since 1.2.0
 * @return Array
 */
function bfwc_get_simple_dates_array( $start_date, $end_date, $buffer ) {
	$return_dates = array();
	$start_date = new DateTime( ( $start_date ) );
	$start_date_ymd = $start_date->format( 'Y-m-d' );
	$end_date = new DateTime( ( $end_date ) );
	$end_date_ymd = $end_date->format( 'Y-m-d' );

	// Iterate through each date from start to end
	// Use $check_date then add one day
	$check_date = new DateTime( $start_date_ymd );
	$check_date = $check_date->format( 'Y-m-d' );
	do {
		if( ! in_array( $check_date, $return_dates ) ) {
			$return_dates[] = $check_date;
		}
		$check_date = new DateTime( $check_date );
		$check_date->modify( '+1 day' );
		$check_date = $check_date->format( 'Y-m-d' );
	} while( $check_date <= $end_date_ymd );

	return $return_dates;
}

/**
 * Get an array of dates to use in the calendar
 * @param $post_id		The product ID or booking ID
 * @param $start_date	The starting date for the list
 * @param $end_date		The end date for the list
 * @param $num_units	Optional number of units, use instead of $end_date
 * @return Array
 */
function bfwc_get_dates_array( $post_id, $start_date, $end_date='', $num_units='' ) {

	// Ensure start and end dates are formatted so that we can read them
	$return_dates = array();
	$unit = bfwc_get_booking_unit( $post_id );
	if( $unit != 'week' ) {
		$unit = 'day';
	}

	if( ! $start_date || ! $end_date ) {
		return array();
	}

	if( ! $num_units ) {
		$new_start_date = new DateTime( ( $start_date ) );
		$date_count = $new_start_date->format( 'Y-m-d' );
		$new_end_date = new DateTime( ( $end_date ) );
		// Can a booking start the same day that another booking ends?
		$buffer = get_post_meta( $post_id, '_buffer', true );
		if( ! $buffer ) $buffer = 0;
		$buffer--;

		$new_end_date = $new_end_date->modify( '+' . $buffer . ' day' );
		$end_date = $new_end_date->format( 'Y-m-d' );

		do {
			$date = new DateTime( $date_count );
			$date_count = $date->format( 'Y-m-d' );
			if( ! in_array( $date_count, $return_dates ) ) {
				$return_dates[] = $date_count;
			}
			$date->modify( '+1 ' . $unit );
			$date_count = $date->format( 'Y-m-d' );
		} while( $date_count <= $end_date );
	} else {
		if( $start_date && $num_units ) {
			// Loop through each day from the start day for the designated number of units
			$date_count = $start_date;
			$duration = bfwc_get_booking_unit_duration( $post_id );
			$total = $num_units * absint( $duration );
			for( $i = 0; $i < $total; $i++ ) {
				$date = new DateTime( date( 'Y-m-d', strtotime( $date_count ) ) );
				$date_count = $date->format( 'Y-m-d' );
				if( ! in_array( $date_count, $return_dates ) ) {
					$return_dates[] = $date_count;
				}
				$date->modify( '+1 day' );
				$date_count = $date->format( 'Y-m-d' );
			}
		}
	}

	return $return_dates;
}

/**
 * Get an array of dates with non-standard costs
 * @param $nonstd_data	The non-standard data for this product
 * @return Array 'dates' => Array of dates, 'costs' => Array of dates=>costs
 */
function bfwc_get_nonstd_dates_array( $nonstd_data ) {
	$nonstd_dates = array();
	foreach( $nonstd_data as $nonstd_item ) {
		// Needs all three parameters to be set
		// Ignore costs set to 0 - these should be blocked out
		if( isset( $nonstd_item['start_date'] ) && isset( $nonstd_item['end_date'] ) && isset( $nonstd_item['cost'] ) && $nonstd_item['cost'] > 0 ) {

			// $date_count = $nonstd_item['start_date'];
			// $end_date = new DateTime( $nonstd_item['end_date'] );

			if( isset( $nonstd_item['start_date_ymd'] ) ) {
				// Added start_date_ymd in 1.3.0 to prevent foreign language dates throwing an error
				$date_count = $nonstd_item['start_date_ymd'];
			} else {
				// Fallback to dates entered pre-1.3.0
				$date_count = $nonstd_item['start_date'];
			}
			if( isset( $nonstd_item['end_date_ymd'] ) ) {
				// Added end_date_ymd in 1.3.0 to prevent foreign language dates throwing an error
				$end_date = new DateTime( $nonstd_item['end_date_ymd'] );
			} else {
				// Fallback to dates entered pre-1.3.0
				$end_date = new DateTime( $nonstd_item['end_date'] );
			}

			$end_date = $end_date->getTimestamp();

			do {
				$date = new DateTime( $date_count );
				$date_count = $date->format( 'Y-m-d' );
				if( ! isset( $nonstd_dates['dates'] ) || ! in_array( $date_count, $nonstd_dates['dates'] ) ) {
					$nonstd_dates['dates'][] = $date_count;
					$nonstd_dates['costs'][$date_count] = $nonstd_item['cost'];
				}
				$date->modify( '+1 day' );
				$date_count = $date->format( 'Y-m-d' );
			} while( $date->getTimestamp() <= $end_date );

		}
	}

	return $nonstd_dates;
}

/**
 * Calculate cost of booking based on selected choices
 */
function bfwc_calculate_booking_cost( $posted ) {

	$product_id = $posted['product_id'];
	$product = wc_get_product( $product_id );

	$duration = bfwc_get_booking_unit_duration( $product_id );
	$minimum = bfwc_get_minimum_duration( $product_id );
	$multiple = absint( $duration ) * absint( $minimum );

	$start = new DateTime( $posted['start_date_ymd'] );

	if( isset( $posted['end_date'] ) ) {
		$end_date = $posted['end_date'];
	}

	$format_end_date = ! empty( $posted['end_date_ymd'] ) ? new DateTime( $posted['end_date_ymd'] ) : '';

	// @since 1.1.4 always calculate num_units using dates
	$num_units = $start->diff( $format_end_date );
	// Use $num_units to calculate cost
	// Calculated initially as a difference in days between start and end
	$num_units = $num_units->days;

	// Get our booking unit, daily, overnight, weekly
	$unit = bfwc_get_booking_unit( $product_id );

	// Add any buffer in to ensure total rounds correctly
	$buffer = get_post_meta( $product_id, '_buffer', true );
	if( ! $buffer ) $buffer = 0;

	// Use $units_with_buffer to work out the end date
	$units_with_buffer = $num_units + $buffer;
	// $num_units += $buffer;

	if( $unit == 'day' ) {
		// Add 1 if we're booking per day, not overnight
		// Need to add one to daily unit to ensure no zero values
		$num_units++;
		$units_with_buffer++;
		$end_date = $start->modify( '+' . ( ( $num_units * absint( $multiple ) ) ) . ' days' );

	} else if( $unit == 'night' ) {

		$end_date = $start->modify( '+' . ( ( $num_units * absint( $multiple ) ) ) . ' days' );

	} else if( $unit == 'week' ) {
		// If the unit is weekly, then $num_units and $units_with_buffer will work out to be the same
		// That's because the buffer is set in days then wrapped into the unit, which is set in weeks
		$num_units = $units_with_buffer / 7;
		$units_with_buffer = $num_units;
		$end_date = $start->modify( '+' . ( ( $units_with_buffer * absint( $multiple ) ) - 1 ) . ' days' );

	}

	// Create array of dates included in booking
	// Will use this to grey out dates on the calendar
	$start_date = $posted['start_date_ymd'];
	if( is_object( $end_date ) ) {
		$end_date = $end_date->format( 'Y-m-d' );
	}
	$pending_dates = bfwc_get_dates_array( $product_id, $start_date, $end_date, $num_units );
	$std_cost = bfwc_get_standard_cost( $product );
	$booking_cost = 0;

	// Check for any custom prices in the booking date
	$nonstd_data = get_post_meta( $product_id, '_nonstd_data', true );

	if( $nonstd_data ) {
		$nonstd_dates = bfwc_get_nonstd_dates_array( $nonstd_data );
		// Cycle through each week of the pending booking to check for custom prices
		// We're checking in blocks, starting at 0
		// If the block matches one of the nonstd blocks, get the nonstd cost
		$count = 0;
		do {
			$check_day = $pending_dates[$count];
			if( isset( $nonstd_dates['dates'] ) && in_array( $check_day, $nonstd_dates['dates'] ) ) {
				$booking_cost += ( float )$nonstd_dates['costs'][$check_day];
			} else {
				$booking_cost += ( float )$std_cost;
			}

			$count += absint( $duration );
		} while( ! empty( $pending_dates[$count] ) );

	} else {
		// There's no non-standard units
		$booking_cost = ( float )$std_cost * $num_units;
		$nonstd_dates = array();
	}

	// Apply a discount if there is one
	$discounts = apply_filters(
		'bfwc_discount_rates',
		get_post_meta( $product_id, '_discount_data', true ),
		$product_id
	);
	$discounted_amount = 0;

	if( $discounts ) {

		// Re-order array to ensure biggest discounts are first
		$key_values = array_column( $discounts, 'after'); 
		array_multisort( $key_values, SORT_DESC, $discounts );

		foreach( $discounts as $discount ) {
			if( ! empty( $discount['rate'] ) && ! empty( $discount['after'] ) && ! empty( $discount['from'] ) ) {
				
				if( $num_units >= $discount['after'] ) {

					// Have we booked enough units for the discount to kick in?
					if( $discount['from'] == 'all' ) {

						// Applying the discount to the entire booking
						$discounted_amount = bfwc_get_discounted_amount( $booking_cost, $discount, $product_id, $num_units );
						$booking_cost = bfwc_get_booking_cost_after_discount( $booking_cost, $discounted_amount, $product_id );
					
					} else if( $discount['from'] == 'partial' ) {

						// Calculate the discount for the part of the booking that qualifies
						$discountable_units = $num_units - $discount['after'];
						$eligible_period = 100 - ( ( ($num_units - $discountable_units ) / $num_units ) * 100 ); // This is the percentage of the booking length that qualifies for discount
						$eligible_booking_cost = $booking_cost * ( $eligible_period / 100 ); // This is the percentage of the booking cost that we'll apply the discount to
						$discounted_amount = number_format( $eligible_booking_cost * ( $discount['rate'] / 100 ), 2 );

						// $booking_cost = ( $booking_cost - $discounted_amount );
						$booking_cost = ( $booking_cost - str_replace( ",", "", $discounted_amount ) );

					}

					// Ensures that discounts aren't cumulative
					break;

				}

			}
		}
	}

	if ( 'incl' === get_option( 'woocommerce_tax_display_shop' ) && is_object( $product ) ) {
		$booking_cost = wc_get_price_including_tax( $product, array( 'price' => $booking_cost ) );
	} else if( is_object( $product ) ) {
		$booking_cost = wc_get_price_excluding_tax( $product, array( 'price' => $booking_cost ) );
	}

	// Multiply cost by quantity if set
	if( ! empty( $posted['num_bookings'] ) ) {
		$booking_cost = $booking_cost * $posted['num_bookings'];
	}

	$return_values = array(
		'product_id' 				=> $product_id,
		'num_units'					=> $num_units,
		'cost'						=> $booking_cost,
		'dates'						=> $pending_dates,
		'nonstd'					=> $nonstd_dates,
		'end'						=> date( 'j F Y', $format_end_date->getTimestamp() ),
		'duration'					=> $duration,
		'discounted_amount' 		=> $discounted_amount
	);

	if ( bfwc_has_aelia() ) {
		// 2.0.8, converted by Aelia if it's activated. We do this here because we have $product object here that we can use
		$booking_cost_converted = bfwc_aelia_get_price( $booking_cost, $product );
		$return_values['cost'] = $booking_cost_converted;
	}

	return $return_values;

}

/**
 * Calculate the discounted amount
 * @since 1.4.8
 */
function bfwc_get_discounted_amount( $booking_cost, $discount, $product_id, $num_units ) {

	$type = bfwc_get_discount_type( $product_id );

	if( $type == 'percentage' ) {
		$discounted_amount = $booking_cost * ( $discount['rate'] / 100 );
	} else if( $type == 'perunit' ) {
		$discounted_amount = ( $discount['rate'] * $num_units );
	} else {
		$discounted_amount = $discount['rate'];
	}

	return $discounted_amount;

}

/**
 * Calculate the price of the booking after any discount is applied
 * @since 1.4.8
 */
function bfwc_get_booking_cost_after_discount( $booking_cost, $discounted_amount, $product_id ) {

	$booking_cost = $booking_cost - $discounted_amount;
	return $booking_cost;

}
						

/**
 * Get the cost of booking based on selected choices
 */
function bfwc_get_booking_cost() {

	$posted = array();

	parse_str( $_POST['form'], $posted );

	if( ! isset( $posted['bookings_nonce'] ) || ! wp_verify_nonce( $posted['bookings_nonce'], 'bookings_nonce' ) ) {
		wp_send_json( array( 'result' => 'error' ) );
		exit;
	}

	$return_values = bfwc_calculate_booking_cost( $posted );
	$lowest_availability = 0;

	// Find the maximum number of available bookings in the given date range
	if( bfwc_get_max_bookings( $posted['product_id'] ) > 1 ) {
		$lowest_availability = bfwc_get_maximum_booking_by_range( $posted['product_id'], $posted['start_date_ymd'], $posted['end_date_ymd'] );
	}

	wp_send_json( array(
		'post_id'			=> $return_values['product_id'],
		'num_units'			=> ceil( $return_values['num_units'] ) ,
		'cost'				=> $return_values['cost'],
		'dates'				=> $return_values['dates'],
		'nonstd'			=> $return_values['nonstd'],
		'end'				=> $return_values['end'],
		'duration'			=> $return_values['duration'],
		'discounted_amount' => $return_values['discounted_amount'],
		'lowest_availability'	=> $lowest_availability
	) );

	exit;

}
add_action( 'wp_ajax_bfwc_get_booking_cost', 'bfwc_get_booking_cost' );
add_action( 'wp_ajax_nopriv_bfwc_get_booking_cost', 'bfwc_get_booking_cost' );

/**
 * Calculate number of available units from selected date
 */
function bfwc_calculate_available_units() {

	$posted = array();

	parse_str( $_POST['form'], $posted );

	if( ! isset( $posted['bookings_nonce'] ) || ! wp_verify_nonce( $posted['bookings_nonce'], 'bookings_nonce' ) ) {
		wp_send_json( array( 'result' => 'error' ) );
		exit;
	}

	$unavailable = '';
	if( isset( $_POST['unavailable'] ) ) {
		$unavailable = $_POST['unavailable'];
	}

	$next = '';
	$available = '';
	$end_date = '';
	$duration = bfwc_get_booking_unit_duration( $posted['product_id'] );
	$minimum = bfwc_get_minimum_duration( $posted['product_id'] ); // $minimum is always at least 1. 0 not allowed
	$maximum = bfwc_get_maximum_duration( $posted['product_id'] );
	$buffer = get_post_meta( $posted['product_id'], '_buffer', true );
	if( ! $buffer ) $buffer = 0;

	$date = $posted['start_date_ymd'];

	// Pass back min_date param for end date calendar
	$format_date = date( 'Y-m-d', strtotime( $date ) ); // Format like this for WP so we can query dates
	$date_format = get_option( 'date_format' ); // no longer in use since 1.4.4
	//$min_date = date( $date_format, strtotime( $date ) ); // Format like this for the datepicker (up to 1.4.3)
	$min_date = $format_date; // since 1.4.4. maybe we don't need to use WP date format since we pass the date back as Y-m-d anyway. Also, strtotime does not recognize d/m/Y properly

	// Set min_date for nightly and weekly bookings
	// The min_date parameter is the first clickable date on the end calendar
	$unit = bfwc_get_booking_unit( $posted['product_id'] );

	$dateString = strstr( $posted['start_date_ymd'], " (", true );

	$start_date_obj = new DateTime( $date );
	$end_date_obj = new DateTime( $date );

	if( $unit == 'night' ) {
		$add_minimum = $start_date_obj->modify( '+' . absint( $minimum ) . ' day' );
	} else if( $unit == 'week' ) {
		// Check for a buffer (set in days even though the unit is weeks)
		$modify = '+' . absint( $minimum ) . ' ' . $unit;
		if( $buffer ) {
			$modify .= ' -' . $buffer . ' day';
		}
		$add_minimum = $start_date_obj->modify( $modify );
	} else {
		// A daily booking can end the same day it starts
		$modification = absint( $minimum ) - 1 . ' ' . $unit;
		$add_minimum = $start_date_obj->modify( '+' . $modification );
	}
	//$min_date = date( $date_format, $add_minimum->getTimestamp() ); // up to 1.4.3, see previous note about WP date format
	$min_date = date( 'Y-m-d', $add_minimum->getTimestamp() ); // since 1.4.4

	$max_date = '';
	if( $maximum ) {
		if( $unit == 'night' ) {
			$modification = absint( $maximum ) . ' day';
			$add_max = $end_date_obj->modify( '+' . $modification );
		} else if( $unit == 'week' ) {
			$modification = absint( $maximum ) . ' weeks';
			$add_max = $end_date_obj->modify( '+' . $modification );
		} else {
			// A daily booking can end the same day it starts
			$modification = absint( $maximum ) - 1 . ' ' . $unit;
			$add_max = $end_date_obj->modify( '+' . $modification );
		}
		//$max_date = date( $date_format, $add_max->getTimestamp() ); // up to 1.4.3, see previous note about WP date format
		$max_date = date( 'Y-m-d', $add_max->getTimestamp() ); // since 1.4.4
	}

	// Used unavailable dates since 1.1.4
	$next = -1;
	$unav_count = 0;
	if( $unavailable ) {

		// Iterate through the array of unavailable dates to find the first unavailable date after selected start date
		$unavailable = explode( ',', $unavailable );

		$start_date_str = strtotime( $posted['start_date_ymd'] );
		foreach( $unavailable as $unavailable_date ) {
			//$start_date_obj = new DateTime( $posted['start_date'] );

			$unav_date_str = strtotime( $unavailable_date );
			if( $unav_date_str > $start_date_str ) {
				if( $buffer == 0 && isset( $unavailable[$unav_count+1] ) ) {
					$next = $unavailable[$unav_count+1];
				} else {
					$next = $unavailable_date;
				}
				if( $unit == 'night' ) {
					// If overnight bookings can overlap, a booking can end on the day that another booking starts
				}
				break;
			}

			$unav_count++;
		}
	}

	$cost = '';
	$set_end_date = '';
	$return_values = array();
	if( ! empty( $posted['end_date'] ) ) {
		// If an end date is present, need to revalidate the dates
		$set_end_date = bfwc_get_valid_end_date( $posted['start_date_ymd'], $posted['end_date_ymd'], $minimum, $maximum, $unit, $posted['product_id'] );
		$return_values = bfwc_calculate_booking_cost( $posted );
		// Try and get costs
		if( isset( $return_values['cost'] ) ) {
			$cost = $return_values['cost'];
		}
	}

	// Return a specific day when a booking must end
	$bookings_start = get_post_meta( $posted['product_id'], '_bookings_start', true );
	$bookings_end = 99;
	if( $unit == 'week' ) {
		if( ! $bookings_start || $bookings_start == 99 ) {
			// If no specific weekday has been set for bookings to start
			// We calculate the end day based on whichever day has just been selected by the user
			$bookings_start = date( 'w', strtotime( $posted['start_date_ymd'] ) );
		}
		// If a specific day has been set
		$bookings_end = $bookings_start - $buffer;
		if( $bookings_end < 0 ) {
			$bookings_end = $bookings_end + 7;
		}
	} else {
		if( $buffer != 0 && ( $bookings_start >= 0 && $bookings_start <=6 ) ) {
			// The booking end day is calculate by subtracting the buffer from the start day
			$bookings_end = $bookings_start - $buffer;
			if( $bookings_end < 0 ) {
				$bookings_end = $bookings_end + 7;
			}
		}
	}

	$discounted_amount = isset( $return_values['discounted_amount'] ) ? $return_values['discounted_amount'] : 0;

	wp_send_json( array(
		// 'num_units'			=> ceil( $num_units ), // Always round up to avoid fractions and to account for buffers
		'date'					=> date( 'j F Y', strtotime( $date ) ),
		'next'					=> $next, // The next booked date - all dates from this date on should be unavailable
		'set_end_date'			=> $set_end_date,
		'available'				=> $available,
		'multi'					=> $duration,
		'min'					=> $minimum,
		'min_date'				=> date( 'Y-m-d', strtotime( $min_date ) ),
		'max_date'				=> date( 'Y-m-d', strtotime( $max_date ) ),
		'cost'					=> $cost,
		'bookings_end'			=> $bookings_end,
		'discounted_amount' 	=> $discounted_amount
	) );

	exit;
}
add_action( 'wp_ajax_bfwc_calculate_available_units', 'bfwc_calculate_available_units' );
add_action( 'wp_ajax_nopriv_bfwc_calculate_available_units', 'bfwc_calculate_available_units' );

/**
 * Checks whether the end date is still valid after the start date has been changed
 * @param $start_date	The new start date displayed in the form
 * @param $end_date		The current end date displayed in the form
 * @param $minimum		Minimum booking duration
 * @param $maximum		Maximum booking duration
 * @param $unit			Booking unit
 * @param $product_id	Product ID
 * @return Mixed		Date if valid, false if not
 */
function bfwc_get_valid_end_date( $start_date, $end_date, $minimum, $maximum, $unit, $product_id ) {
	$valid_end_date = $end_date;
	$unavailable_dates = bfwc_get_unavailable_dates( $product_id );

	if( in_array( $end_date, $unavailable_dates ) ) {
		return false;
	}
	// Check if end - start is less than minimum
	$diff = bfwc_get_date_difference( $start_date, $end_date, $unit );

	if( $diff < $minimum ) {
		return false;
	}
	return $diff;
}

/**
 * Create a new booking when a user adds it to their cart
 * Updated 1.3.4, first param changed from $cart_item_meta to $cart_item_meta['booking']
 * @param $status Set to pending by default as booking is not yet paid for
 * @param $booking_data cart or order item meta specific to the booking
 */
function bfwc_create_new_booking( $booking_meta, $product_id, $status="pending" ) {

	if( isset( $booking_meta['num_units'] ) ) {
		$num_units = absint( $booking_meta['num_units'] );
	}
	if( isset( $booking_meta['cost'] ) ) {
		$cost = absint( $booking_meta['cost'] );
	}

	// We need to check if a booking has already been made with these dates
	// A user could make a selection then pause before adding to cart
	// Another user could create a booking in that pause
	$can_book = bfwc_is_booking_allowed( $booking_meta['start_date'], $booking_meta['end_date'], $product_id );
	if( ! $can_book ) {
		// Add notification
		wc_add_notice( apply_filters( 'bfwc_filter_booking_not_permitted', __( 'That booking is not permitted. Please try again.' ) ), 'error' );
		return false;
	}
	$booking_id = wp_insert_post( array(
		'post_title'	=> __( 'Draft booking', 'bfwc' ),
		'post_type'   	=> 'bfwc_booking',
		'post_status'	=> 'publish'
	) );

	// Update the booking title
	$title_str = bfwc_create_booking_title( $booking_id, $product_id, $booking_meta['start_date_text'], $booking_meta['end_date_text'] );
	$update = array(
		'ID'			=> $booking_id,
		'post_title'	=> $title_str
	);
	update_post_meta( $booking_id, 'bfwc_booking_title', $title_str );
	wp_update_post( $update );

	update_post_meta( $booking_id, 'bfwc_product_id', absint( $product_id ) );
	$title = get_the_title( $product_id );
	update_post_meta( $booking_id, 'bfwc_booking_product', sanitize_text_field( $title ) );
	if( isset( $booking_meta['num_units'] ) ) {
		update_post_meta( $booking_id, 'bfwc_num_units', absint( $booking_meta['num_units'] ) );
	}
	if( isset( $booking_meta['start_date'] ) ) {
		update_post_meta( $booking_id, 'bfwc_start_date', esc_attr( $booking_meta['start_date'] ) );
	}
	if( isset( $booking_meta['start_date_text'] ) ) {
		update_post_meta( $booking_id, 'bfwc_start_date_text', esc_attr( $booking_meta['start_date_text'] ) );
	}
	if( isset( $booking_meta['buffer'] ) ) {
		update_post_meta( $booking_id, 'bfwc_buffer', esc_attr( $booking_meta['buffer'] ) );
	}
	if( isset( $booking_meta['booking_unit'] ) ) {
		update_post_meta( $booking_id, 'bfwc_booking_unit', sanitize_text_field( $booking_meta['booking_unit'] ) );
	}
	if( isset( $booking_meta['num_bookings'] ) ) {
		update_post_meta( $booking_id, 'bfwc_num_bookings', sanitize_text_field( $booking_meta['num_bookings'] ) );
	}
	if( isset( $booking_meta['num_bookings'] ) ) {
		update_post_meta( $booking_id, 'bfwc_quantity', sanitize_text_field( $booking_meta['num_bookings'] ) );
	}

	// This is the end date without the buffer
	// It's used to block out availability for future bookings
	if( isset( $booking_meta['end_date'] ) ) {
		update_post_meta( $booking_id, 'bfwc_end_date', esc_attr( $booking_meta['end_date'] ) );
	}
	// This is the end date with the buffer
	// It's not used elsewhere but is helpful to see when the booking finishes
	// This will be the same date as the end_date for weekly bookings as the buffer is wrapped inside weekly bookings
	if( isset( $booking_meta['end_date_plus_buffer'] ) ) {
		update_post_meta( $booking_id, 'bfwc_end_date_plus_buffer', esc_attr( $booking_meta['end_date_plus_buffer'] ) );
	}
	if( isset( $booking_meta['end_date_text'] ) ) {
		update_post_meta( $booking_id, 'bfwc_end_date_text', esc_attr( $booking_meta['end_date_text'] ) );
	}
	if( isset( $booking_meta['cost'] ) ) {
		update_post_meta( $booking_id, 'bfwc_cost', absint( $booking_meta['cost'] ) );
	}
	update_post_meta( $booking_id, 'bfwc_status', $status );

	// Update the Booked days post type for each booked unit - used to track the number of bookings used per product per booking unit
	bfwc_update_booked_days( $product_id, $booking_id, $booking_meta );

	if( bfwc_get_block_dates_after_order() == 'no' ) {
		// If bookings are created as soon as products are added to the cart
		// Schedule them for deletion
		$minutes = bfwc_get_cart_expiry();
		$timestamp = time() + MINUTE_IN_SECONDS * absint( $minutes );
		wp_schedule_single_event( $timestamp, 'bfwc_check_pending_bookings', array( $booking_id ) );
	}

	do_action( 'bfwc_after_create_new_booking', $booking_id, $status, $booking_meta, $product_id );

	return $booking_id;

}

/**
 * Cancel bookings where a user hasn't checked out in the required time
 * @todo - check wc_cancel_unpaid_orders for comparison
 * @since 2.0.0
 */
function bfwc_maybe_cancel_pending_booking( $booking_id ) {	

	if( bfwc_get_block_dates_after_order() == 'no' ) {
		
		// If this booking still has a status of 'Pending', we need to delete it
		if( get_post_type( $booking_id ) == 'bfwc_booking' ) {

			$status = get_post_meta( $booking_id, 'bfwc_status', true );
			if( $status == 'pending' || $status == 'removed' ) {

				// Need to restore booking dates
				bfwc_restore_booked_days_by_booking_id( $booking_id );

				// Delete the booking
				wp_delete_post( $booking_id );
				
			}

		}
		
	}

}
add_action( 'bfwc_check_pending_bookings', 'bfwc_maybe_cancel_pending_booking' );

/**
 * We need to validate the order to check that the booking has not been deleted (e.g. after 30 mins)
 * @since 2.0.0
 */
function bfwc_checkout_process( $fields, $errors ) {

	// Look for booking status
	foreach ( WC()->cart->get_cart() as $cart_item_key=>$cart_item ) {

		if( ! empty( $cart_item['booking']['_booking_id'] ) ) {

			// Check if the booking has been deleted
			$status = get_post_status( $cart_item['booking']['_booking_id'] );
			if( $status != 'publish' ) {

				// If the booking has been deleted, remove the product from the cart
				WC()->cart->remove_cart_item( $cart_item_key );
				// Let the customer know
				$errors->add( 'bfwc_checkout', __( 'Your booking has been automatically removed from the cart', 'bfwc' ) );

			}
			
		}

	}

}
add_action( 'woocommerce_after_checkout_validation', 'bfwc_checkout_process', 10, 2 );

/**
 * Create a title for a booking using product ID, start and end dates
 * @since 2.0.0
 */
function bfwc_create_booking_title( $booking_id, $product_id, $start_date, $end_date ) {

	$product = wc_get_product( $product_id );
	if( ! is_object( $product ) ) {
		return __( 'Draft booking', 'bfwc' );
	}
	$title_str = '&#x23;' . $booking_id . '&nbsp;';
	$title_str .= $product->get_title() . ': ';
	if( $start_date ) {
		$title_str .= __( 'From ', 'bfwc' ) . esc_attr( apply_filters( 'bfwc_booking_title_start_date', $start_date ) );
	}
	if( $end_date ) {
		$title_str .= __( ' to ', 'bfwc' ) . esc_attr( apply_filters( 'bfwc_booking_title_end_date', $end_date ) );
	}

	return $title_str;

}

/**
 * Format the date string in the booking title to match the site date format
 * @since 2.0.0
 */
function bfwc_format_booking_title_date( $date ) {

	$date_format = get_option( 'date_format' );
	$formatted_date = bfwc_date_time( $date );
	$formatted_date = $formatted_date->format( $date_format );

	return $formatted_date;

}
// add_filter( 'bfwc_booking_title_start_date', 'bfwc_format_booking_title_date' );
// add_filter( 'bfwc_booking_title_end_date', 'bfwc_format_booking_title_date' );

/**
 * Check that the booking we're trying to create doesn't break any rules
 * @param $start_date
 * @param $end_date
 * @param $product_id
 * @return Boolean
 */
function bfwc_is_booking_allowed( $start_date, $end_date, $product_id ) {

	$duration = bfwc_get_booking_unit_duration( $product_id );
	$minimum = bfwc_get_minimum_duration( $product_id );

	$start_date = date( 'Y-m-d', strtotime( $start_date ) );
	$end_date = date( 'Y-m-d', strtotime( $end_date ) );

	// Get the dates between start and stop, inclusive
	// Query each date to see if it overlaps an existing booking

	// Do a query to find if any bookings exist within these dates
	$args = array(
		'post_type'			=> 'bfwc_booking',
		'posts_per_page'	=> 1,
		'orderby'			=> 'meta_value',
		'meta_key'			=> 'bfwc_start_date',
		'order'				=> 'ASC',
		'meta_query'		=> array(
			'relation'		=> 'AND',
			array(
				'key'		=> 'bfwc_start_date',
				'value'		=> $start_date,
				'compare'	=> '>=',
				'type'		=> 'DATE'
			),
			array(
				'key'		=> 'bfwc_start_date',
				'value'		=> $end_date,
				'compare'	=> '<',
				'type'		=> 'DATE'
			),
			array(
				'key'		=> 'bfwc_status',
				'value'		=> 'removed',
				'compare'	=> '!='
			),
			array(
				'key'		=> 'bfwc_product_id',
				'value'		=> $product_id,
				'compare'	=> '='
			)
		),
		'fields'			=> 'ids' // Just return the IDs
	);
	$bookings = new WP_Query( $args );

	// If there are bookings, iterate through each one
	if( $bookings->have_posts() ) {

		// If there are bookings, we have to iterate through each day to check if there are still available slots
		$max_bookings = bfwc_get_max_bookings( $product_id );

		// Only need to do this if there are multiple booking slots for this product
		if( $max_bookings > 1 ) {
			// If any of the days does not have an available slot, the booking cannot be made
			return bfwc_check_bookings_in_range( $start_date, $end_date, $product_id, $max_bookings );
		}

		return false;

	}

	return true;

}

/**
 * Delete a pending or removed booking after 30 mins
 * Deprecated @since 2.0.0
 * Use bfwc_maybe_cancel_pending_booking instead
 */
function bfwc_delete_pending_booking( $booking_id ) {
	
	if ( $booking_id ) {
		$status = get_post_meta( $booking_id, 'bfwc_status', true );
		if( $status == 'pending' || $status == 'removed' ) {
			wp_delete_post( $booking_id );
		}
	}
}
add_action( 'bfwc_delete_pending_booking', 'bfwc_delete_pending_booking' );

/**
 * Wrapper for new DateTime to ensure non-English dates are parsed okay
 * @since 2.0.0
 */
function bfwc_date_time( $date ) {

	if( is_object( $date ) ) {
		// Already a date object
		return $date;
	}

	$date = bfwc_get_ymd( $date );
	$date = new DateTime( $date );

	return $date;

}

/**
 * Get a date in YMD format
 * @since 2.0.0
 */
function bfwc_get_ymd( $date ) {

	// Transpose month and day?
	$date = bfwc_transpose_month_day( $date );

	// Convert this to English if needed
	$date = bfwc_get_date_from_locale( $date );

	$date = new DateTime( $date );
	$date_ymd = $date->format( 'Y-m-d' );
	return $date_ymd;

}

/**
 * Ensure d/m/y dates are presented as m/d/y
 * @since 2.0.1
 */
function bfwc_transpose_month_day( $date ) {

	// If $date doesn't have / as a separator, then it's already formatted as we want
	$first = strpos( $date, '/' );
	if( $first === false ) {
		return $date;
	}

	$format = get_option( 'date_format' );
	// Ensure we use d to find the day
	$format = str_replace( 'j', 'd', $format );

	$d = strpos( $format, 'd' );
	$m = strpos( $format, 'm' );

	// If the format places day before month, we need to switch them around
	if( false !== $d && false !== $m && $d < $m ) {

		$last = strripos( $date, '/', -1 );
		$day = substr( $date, 0, $first );
		$month = str_replace( '/', '', substr( $date, $m + 1, $last - $first ) );
		
		$year = substr( $date, $last + 1, strlen( $date ) );
		
		// We need to transpose these two elements
		$date = $month . '/' . $day . '/' . $year;

	} else {

		// 2.0.4
		list( $d, $m, $y ) = explode( '/', $date ); // let's assume 
		if ( $d && $m && $y && $d > $m && checkdate( $m, $d, $y ) ) {
			// this was valid, so transpose
			$date = $m . '/' . $d . '/' . $y;
		}

	}
	
	return $date;

}

/**
 * Get the English date from the locale date
 * There has to be a better way than this (not sure about date_i18n)
 * @since 2.0.0
 */
function bfwc_get_date_from_locale( $date ) {

	$new_date = '';
	$lang = get_bloginfo( 'language' );
	$locale = str_replace( '-', '_', $lang );

	// Format the date based on our language/locale
	$dateFormatter = new IntlDateFormatter(
		$locale,
		IntlDateFormatter::LONG, // date type
		IntlDateFormatter::NONE  // time type
	);
	// Full month name
	$dateFormatter->setPattern( 'LLLL' );

	$english_months = array(
		'1'		=> 'January',
		'2'		=> 'February',
		'3'		=> 'March',
		'4'		=> 'April',
		'5'		=> 'May',
		'6'		=> 'June',
		'7'		=> 'July',
		'8'		=> 'August',
		'9'		=> 'September',
		'10'	=> 'October',
		'11'	=> 'November',
		'12'	=> 'December',
	);

	$months_locale = [];
	// Iterate through each month looking for a match with the English version
	for( $i = 1; $i <= 12; $i++ ) {
		$month = $dateFormatter->format(
			// 'n' => month number with no leading zeros
			DateTime::createFromFormat( 'n', ( string ) $i )
		);
		// If we find our month, replace it with the English version
		if( gettype( $date ) == 'string' && str_contains( strtolower( $date ), strtolower( $month ) ) ) {
			$new_date = str_replace( strtolower( $month ), $english_months[$i], strtolower( $date ) );
			return $new_date;
		}
	}

	// Return the new date
	return $date;

}

function bfwc_error_log( $var ) {
	if( is_array( $var ) ) {
		$var = print_r( $var, true );
		error_log( $var );
	} else if( is_object( $var ) ) {
		$var = print_r( $var, true );
		error_log( $var );
	} else {
		error_log( $var );
	}
}
