<?php
/**
 * Functions for the cart
 * @since 1.0.0
 * @package Bookings for WooCommerce
 */

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Add to cart template for simple bookings
 */
function bfwc_add_to_cart() {
	// global $product;
	// Get template
	wc_get_template( 'templates/booking.php', array(), 'bfwc', BFWC_PLUGIN_DIR_PATH );
}
add_action( 'woocommerce_simple_booking_add_to_cart', 'bfwc_add_to_cart', 30 );

/**
 * Create a new Booking post from the cart data
 */
function bwfc_add_cart_item_data( $cart_item_meta, $product_id ) {

	$product = wc_get_product( $product_id );

	// If this isn't a booking, we can leave at this point
	if( ! $product->is_type( 'simple_booking' ) ) {
		return $cart_item_meta;
	}

	// Format start date as YYYY-mm-dd
	$start_date = date( 'Y-m-d', strtotime( $_POST['start_date_ymd'] ) );
	$end_date = date( 'Y-m-d', strtotime( $_POST['end_date_ymd'] ) );

	$start_date_text = $_POST['start_date'];
	$end_date_text = $_POST['end_date'];

	$cart_item_meta['booking'] = array(
		'start_date' 				=> $start_date,
		'end_date' 					=> $end_date,
		'start_date_text' 	=> $start_date_text,
		'end_date_text' 		=> $end_date_text,
		'cost' 							=> apply_filters( 'bfwc_filter_calculated_booking_cost', $_POST['calculated_booking_cost'], $_POST )
	);

	if( isset( $_POST['num_units'] ) ) {
		$cart_item_meta['booking']['num_units'] = absint( $_POST['num_units'] );
	}
	if( isset( $_POST['num_units_int'] ) ) {
		$cart_item_meta['booking']['num_units_int'] = absint( $_POST['num_units_int'] );
	}
	if( isset( $_POST['num_bookings'] ) ) {
		$cart_item_meta['booking']['num_bookings'] = absint( $_POST['num_bookings'] );
	} else {
		$cart_item_meta['booking']['num_bookings'] = apply_filters( 'bfwc_default_num_bookings', 1, $product_id ); // 2.0.4. num_bookings does not exist on the product page if bfwc_get_max_bookings() is not set
	}

	/**
	 * Add the buffer and booking unit
	 * @since 1.3.0
	 */
	$unit = bfwc_get_booking_unit( $product_id );
	$buffer = get_post_meta( $product_id, '_buffer', true );
	$cart_item_meta['booking']['end_date'] = $end_date;
	$cart_item_meta['booking']['end_date_plus_buffer'] = $end_date;

 	if( $buffer ) {
		// If there's a buffer, add it to the end date
		// In order to ensure correct dates are blocked for future bookings
		// Only for daily and overnight bookings
		if( $unit != 'weekly' ) {
			$end_date_obj = new DateTime( $end_date );
			$end_date_plus_buffer = $end_date_obj->modify( '+' . absint( $buffer ) . ' day' );
			$end_date_plus_buffer = date( 'Y-m-d', $end_date_plus_buffer->getTimestamp() );
			$cart_item_meta['booking']['end_date_plus_buffer'] = $end_date_plus_buffer;
		}
	} else {
		$buffer = 0;
	}

	$cart_item_meta['booking']['buffer'] = absint( $buffer );
	$cart_item_meta['booking']['booking_unit'] = sanitize_text_field( $unit );

	// Check we can create the new booking
	$can_book = bfwc_is_booking_allowed( $start_date, $end_date, $product_id );
	if( ! $can_book ) {
		// Add notification
		wc_add_notice( apply_filters( 'bfwc_filter_validation_fail_booking_exists', __( 'Your booking could not be validated. Please try again.', 'bfwc' ) ), 'error' );
		return false;
	}

	// Create a new booking
	if( bfwc_get_block_dates_after_order() == 'no' ) {
		// Added 1.3.4, create a booking on add to cart unless bfwc_block_after_order is enabled
		// First param changed from sending all cart_item_meta to just sending the booking element
		$booking_id = bfwc_create_new_booking( $cart_item_meta['booking'], $product_id );
		// Store in cart
		if( $booking_id ) {
			$cart_item_meta['booking']['_booking_id'] = $booking_id;
		}
	}

	return $cart_item_meta;

}
add_filter( 'woocommerce_add_cart_item_data', 'bwfc_add_cart_item_data', 10, 2 );

/**
 * Set the price here
 */
function bfwc_add_cart_item( $cart_item ) {
	$pewc_price = 0;
	// remove pewc filter on this
	if ( ! empty( $cart_item['product_extras'] ) && ! empty( $cart_item['product_extras']['price_with_extras'] ) ) {
		$pewc_price = $cart_item['product_extras']['price_with_extras'];
	}

	if ( ! empty( $cart_item['booking'] ) && ! empty( $cart_item['booking']['cost'] ) ) {
		$booking_price = apply_filters( 'bfwc_filter_booking_cost_add_cart_item', $cart_item['booking']['cost'], $cart_item );
		$cart_item['data']->set_price( $booking_price );
	}
	return $cart_item;
}
add_filter( 'woocommerce_add_cart_item', 'bfwc_add_cart_item', 10, 1 );

/**
 * Filter added to cart message to include button to change booking
 */
function bfwc_add_to_cart_message_html( $message, $products ) {
	// Check if $products contain a booking product
	$is_booking = false;
	foreach( $products as $product_id=>$value ) {
		$product = wc_get_product( $product_id );
		if( $product->is_type( 'simple_booking' ) ) {
			$is_booking = true;
			break;
		}
	}
	if( $is_booking ) {
		$added_text = apply_filters( 'bfwc_filter_added_to_cart_message', __( 'Your booking has been added to your cart.', 'bfwc' ) );
		$message = sprintf(
			'<a href="%s" class="button wc-forward">%s</a> %s',
			esc_url( wc_get_page_permalink( 'cart' ) ),
			esc_html__( 'View cart', 'bfwc' ),
			esc_html( $added_text )
		);
		$message .= sprintf(
			'<a style="margin-right: 1em;" href="%s" class="button change-booking">%s</a>',
			'#',
			esc_html__( 'Change booking', 'bfwc' )
		);
	}
	return $message;
}
add_filter( 'wc_add_to_cart_message_html', 'bfwc_add_to_cart_message_html', 10, 2 );

/**
 * Remove current booking from cart - triggered by selecting the 'Change Booking' button on the View Cart notice
 */
function bfwc_change_booking() {
	$posted = array();

	parse_str( $_POST['form'], $posted );

	if( ! isset( $posted['bookings_nonce'] ) || ! wp_verify_nonce( $posted['bookings_nonce'], 'bookings_nonce' ) ) {
		wp_send_json( array( 'result' => 'error' ) );
		exit;
	}

	// Get the product ID and cart item ID
  	global $woocommerce;
	$status = 'failed';
	$product_id = $posted['product_id'];
	$url = $posted['permalink'];
	// Get all items in cart
	foreach ( $woocommerce->cart->get_cart() as $cart_item_key => $values ) {
		$_product = $values['data'];
		// If the current product is in the cart
		if( $product_id == $_product->get_id() ) {
			$woocommerce->cart->remove_cart_item( $cart_item_key );
			// Get the booking_id and delete the booking
			$cart_item = $woocommerce->cart->removed_cart_contents[$cart_item_key];
			if ( isset( $cart_item['booking']['_booking_id'] ) ) {
				$booking_id = $cart_item['booking']['_booking_id'];
				wp_delete_post( $booking_id );
			}
			$status = 'removed';
		}
	}

	wp_send_json( array(
		'status'	=> $status,
		'url'		=> esc_url( $url )
	) );

	exit;

}
add_action( 'wp_ajax_bfwc_change_booking', 'bfwc_change_booking' );
add_action( 'wp_ajax_nopriv_bfwc_change_booking', 'bfwc_change_booking' );

/**
 * Check whether we can add the booking to the cart
 * Perhaps someone else has sneaked in and booked these dates
 */
function bfwc_validate_cart_item( $passed, $product_id, $quantity ) {

	$product = wc_get_product( $product_id );

	// If this isn't a booking, we can leave at this point
	if( ! $product->is_type( 'simple_booking' ) ) {
		return true;
	}
	$allow_zero_cost = apply_filters( 'bfwc_allow_zero_cost_bookings', false );
	if( empty( $_POST['start_date'] ) || empty( $_POST['end_date'] ) || ( empty( $_POST['calculated_booking_cost'] ) && ! $allow_zero_cost ) ) {
		// Add notification
		wc_add_notice( apply_filters( 'bfwc_filter_validation_fail_parameters', __( 'Your booking is missing some information. Please check and try again.', 'bfwc' ) ), 'error' );
		return false;
	}

	// 2.0.4, we can only have 1 booking product in the cart at a time
	if ( bfwc_booking_products_in_cart() >= apply_filters( 'bfwc_max_bookings_in_cart', 1 ) ) {
		return false;
	}

	// Format start date as YYYY-mm-dd
	$start_date = bfwc_get_ymd( $_POST['start_date'] );
	$end_date = bfwc_get_ymd( $_POST['end_date'] );

	$can_book = bfwc_is_booking_allowed( $start_date, $end_date, $product_id );
	if( ! $can_book ) {
		// Add notification
		wc_add_notice( apply_filters( 'bfwc_filter_validation_fail_booking_exists', __( 'A booking with that date already exists. Please try again.', 'bfwc' ) ), 'error' );
		// Delete the booking
		if( isset( $cart_item['booking']['_booking_id'] ) ) {
			$booking_id = $cart_item['booking']['_booking_id'];
			wp_delete_post( $booking_id );
		}

		return false;
	}

	return true;

}
add_filter( 'woocommerce_add_to_cart_validation', 'bfwc_validate_cart_item', 10, 3 );

/**
 * Update item price with calculated price
 */
function bfwc_booking_item_price( $cart_object ) {
  foreach ( $cart_object->get_cart() as $key=>$value ) {
		if( isset( $value['booking']['cost'] ) ) {
			$price = $value['booking']['cost'];
			$value['data']->set_price( $price );
		}
  }
	return $cart_object;
}
add_action( 'woocommerce_before_calculate_totals', 'bfwc_booking_item_price', 10 );

/**
 * Display custom fields for each cart item
 */
function bfwc_get_item_data( $other_data, $cart_item ) {
	$date_format = get_option( 'date_format' );
	$labels = apply_filters(
		'bfwc_filter_custom_meta_cart_labels_for',
		array(
			'start_date_text'	=> __( 'Starts', 'bfwc' ),
			'end_date_text'		=> __( 'Ends', 'bfwc' ),
			'num_bookings'		=> apply_filters( 'bfwc_quantity_bookings_text', __( 'Quantity', 'bfwc' ), false )
		)
	);
	if ( ! empty( $cart_item['booking'] ) ) {
		foreach ( $cart_item['booking'] as $key => $value ) {
			if ( ! empty( $labels[$key] ) ) {
				$other_data[] = array(
					'name'    => $labels[$key],
					'value'   => $value,
					'display' => '',
				);
			}
		}
	}
	return $other_data;
}
add_filter( 'woocommerce_get_item_data', 'bfwc_get_item_data', 10, 2 );

/**
 * Filter cart to disable quantity input field
 */
function bfwc_remove_quantity_input_cart( $product_quantity, $cart_item_key, $cart_item ) {
	// Check the product is a booking
	if ( ! empty( $cart_item['booking'] ) ) {
		return '1';
	}
  return $product_quantity;
}
add_filter( 'woocommerce_cart_item_quantity', 'bfwc_remove_quantity_input_cart', 10, 3 );

/**
 * Change status of booking when removed from cart
 */
function bfwc_cart_item_removed( $cart_item_key ) {
	$cart_item = WC()->cart->removed_cart_contents[ $cart_item_key ];
	if ( isset( $cart_item['booking']['_booking_id'] ) ) {
		$booking_id = $cart_item['booking']['_booking_id'];
		update_post_meta( $booking_id, 'bfwc_status', 'removed' );
		// restore availability
		$product_id = $cart_item['product_id'];
		$booking_meta = array(
			'start_date' => $cart_item['booking']['start_date'],
			'end_date' => $cart_item['booking']['end_date'],
			'num_bookings' => $cart_item['booking']['num_bookings'] ?? 1
		);
		bfwc_update_booked_days( $product_id, $booking_id, $booking_meta, 'restore' );
	}
}
add_action( 'woocommerce_cart_item_removed', 'bfwc_cart_item_removed' );

/**
 * Change status of booking when restored to cart
 */
function bfwc_cart_item_restored( $cart_item_key ) {
	$cart = WC()->cart->get_cart();
	$cart_item = $cart[ $cart_item_key ];

	if ( isset( $cart_item['booking']['_booking_id'] ) ) {
		$booking_id = $cart_item['booking']['_booking_id'];
		// Need to check if the dates are still available
		if( isset( $cart_item['booking']['end_date'] ) && isset( $cart_item['booking']['start_date'] ) ) {
			// Check that this booking is still valid
			$can_book = bfwc_is_booking_allowed( $cart_item['booking']['start_date'], $cart_item['booking']['end_date'], $cart_item['product_id'] );
			if( $can_book ) {
				// If still valid, restore
				update_post_meta( $booking_id, 'bfwc_status', 'pending' );
			} else {
				// Delete this booking
				wp_delete_post( $booking_id );
			}
		}
	}
}
add_action( 'woocommerce_cart_item_restored', 'bfwc_cart_item_restored' );

/**
 * Empty the user's cart when they log out
 * @since 1.2.1
 */
function bfwc_empty_cart() {
	if( function_exists( 'WC' ) ) {
		WC()->cart->empty_cart();
	}
}
add_action( 'wp_logout', 'bfwc_empty_cart' );

/**
 * Prevent more than 1 booking products in cart
 * @since 2.0.4
 */
function bfwc_booking_products_in_cart() {

	$cart = WC()->cart->get_cart();
	$num_bookings = 0;

	if ( ! empty( $cart ) ) {
		foreach ( $cart as $cart_item_key => $cart_item_data ) {
			if ( $cart_item_data['data']->is_type( 'simple_booking') ) {
				$num_bookings++;
				if ( $num_bookings > 1 ) {
					// we only allow 1 booking product in the cart, remove this booking product now
					WC()->cart->remove_cart_item( $cart_item_key );
				}
			}
		}

		if ( $num_bookings > apply_filters( 'bfwc_max_bookings_in_cart', 1 ) ) {
			// we already have a booking product in the cart
			$message = apply_filters( 'bfwc_filter_validation_fail_booking_product_already_exists', __( 'You can only have one booking product in your cart.', 'bfwc' ) );
			if ( $num_bookings > 1 ) {
				// there were more than 1 booking products, so some were removed, notify customer
				$message .= ' '.apply_filters( 'bfwc_filter_validation_fail_more_than_1_bookings', __( 'The other booking products have been removed from your cart.', 'bfwc' ) );
			}
			$message .= ' '.sprintf(
				'<a href="%s" class="button wc-forward">%s</a>',
				esc_url( wc_get_page_permalink( 'cart' ) ),
				esc_html__( 'View cart', 'bfwc' ),
			);
			wc_add_notice( $message, 'error' );
		}
	}

	return $num_bookings;
}
