<?php
/**
 * Functions for general stuff
 * @since 1.4.0
 * @package Bookings for WooCommerce
 */

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}

use Automattic\WooCommerce\Utilities\OrderUtil;

/**
 * Get the booking unit
 */
function bfwc_get_booking_unit( $product_id=false ) {
	if( ! $product_id ) {
		$product_id = get_the_ID();
	}
	$booking_unit = get_post_meta( $product_id, '_booking_unit', true );
	return apply_filters( 'bfwc_booking_unit', $booking_unit, $product_id );
}

/**
 * Get either day or week for unit
 * @return String
 */
function bfwc_get_booking_unit_day_week( $post_id='' ) {
	if( empty( $post_id ) ) {
		$post_id = get_the_ID();
	}
	$booking_unit = get_post_meta( $post_id, '_booking_unit', true );
	if( $booking_unit == 'week' ) {
		return 'week';
	}
	return 'day';
}

/**
 * Get the booking unit duration in days, e.g. 1 for daily, 7 for weekly
 * @return Integer
 */
function bfwc_get_booking_unit_duration( $post_id='' ) {
	$booking_unit = bfwc_get_booking_unit( $post_id );
	$duration = 1;
	if( $booking_unit == 'week' ) {
		$duration = 7;
	}
	return $duration;
}

/**
 * Get the minimum booking unit duration
 * @return Integer
 */
function bfwc_get_minimum_duration( $post_id='' ) {
	$minimum = get_post_meta( $post_id, '_min_length', true );
	if( ! $minimum || $minimum < 1 ) {
		$minimum = 1;
	}
	return absint( $minimum );
}

/**
 * Get the maximum booking unit duration
 * @return Integer
 */
function bfwc_get_maximum_duration( $post_id='' ) {
	$maximum = get_post_meta( $post_id, '_max_length', true );
	if( ! $maximum || $maximum < 1 ) {
		$maximum = 9999;
	}
	return absint( $maximum );
}

/**
 * Get whether to display nice colours in calendar
 * @since 1.4.0
 */
function bfwc_get_optimise_colours() {
  $optimise = get_option( 'bfwc_optimise_colours', 'no' );
  return $optimise;
}

/**
 * Get whether to display unit availability on hover
 * @since 1.4.0
 */
function bfwc_get_show_availability() {
  $show = get_option( 'bfwc_show_availability', 'no' );
  return $show;
}

/**
 * Get whether to display prices by date on calendars
 * @since 2.0.0
 */
function bfwc_get_show_prices() {
  $show = get_option( 'bfwc_show_prices', 'no' );
  return $show;
}

/**
 * Get whether to display search field on archive pages
 * @since 2.0.0
 */
function bfwc_get_archive_search() {
	$show = get_option( 'bfwc_archive_search', 'no' );
	return $show;
  }

/** Get the discount type
 * @since 1.4.8
 */
function bfwc_get_discount_type( $product_id ) {
	return apply_filters( 'bfwc_discount_type', 'percentage', $product_id );
}

/**
 * Get the maximum booking unit duration
 * @param $start
 * @param $end
 * @param $unit	Either day, night or week
 * @return Integer
 */
function bfwc_get_date_difference( $start, $end, $unit ) {
	$start = new DateTime( $start );
	$end = new DateTime( $end );
	$interval = $start->diff( $end );
	if( $unit == 'week' ) {
		$difference = (int)$interval->days;
		$difference = $difference / 7;
	} else if( $unit == 'day' ) {
		$difference = (int)$interval->days;
		// Allow for same day bookings
		$difference++;
	} else {
		$difference = (int)$interval->days;
	}
	return $difference;
}

/**
 * Get the number of bookings that can be booked per product, e.g. three identical rooms, three hot tubs
 * @param $product_id
 * @return Integer
 * @since 1.4.0
 */
function bfwc_get_max_bookings( $product_id ) {
	$max_bookings = get_post_meta( $product_id, '_max_bookings', true );
	if( ! $max_bookings ) $max_bookings = 1;
	return $max_bookings;
}

/**
 * Can the user book multiple instances of a product?
 * @param $product_id
 * @return Integer
 * @since 1.4.0
 */
function bfwc_get_book_multiple( $product_id ) {
	$book_multiple = bfwc_get_max_bookings( $product_id ) > 1;
	return $book_multiple;
}

/**
 * Get the allow overlaps setting for a specific product
 * @since 1.4.0
 */
function bfwc_get_allow_overlaps( $product_id ) {

  $allow_overlaps = get_post_meta( $product_id, '_allow_overlaps', true );
  return apply_filters( 'bfwc_allow_overlaps', $allow_overlaps, $product_id );

}

/**
 * Check whether a product is is overlappable, i.e. is overnight and allows overlaps
 * @since 1.4.0
 */
function bfwc_is_overlappable( $product_id ) {

  $allow_overlaps = bfwc_get_allow_overlaps( $product_id );
  $booking_unit = bfwc_get_booking_unit( $product_id );
  $is_overlappable = ( $allow_overlaps == 'yes' && $booking_unit == 'night' ) ? 'yes' : 'no';
  return apply_filters( 'bfwc_is_overlappable', $is_overlappable, $product_id );

}

/**
 * WordPress has deprecated get_page_by_title since 6.2, so let's create our own function
 * @since 1.4.8
 */
function bfwc_get_page_by_title( $page_title, $post_type ) {

	$query = new WP_Query(
		array(
			'post_type'              => $post_type,
			'title'                  => $page_title,
			'post_status'            => 'all',
			'posts_per_page'         => 1,
			'no_found_rows'          => true,
			'ignore_sticky_posts'    => true,
			'update_post_term_cache' => false,
			'update_post_meta_cache' => false,
			'orderby'                => 'post_date ID',
			'order'                  => 'ASC',
		)
	);

	if ( ! empty( $query->post ) ) {
		$page_got_by_title = $query->post;
	} else {
		$page_got_by_title = null;
	}

	return $page_got_by_title;

}

/**
 * Get a list of all user roles on the site
 * @since 1.4.8
 */
function bfwc_get_user_roles() {
	global $wp_roles;
	$roles = $wp_roles->role_names;
	$options = array(
		''	=> '',
	);
	if( $roles ) {
		foreach( $roles as $id=>$role ) {
			$options[$id] = $role;
		}
	}
	return $options;
}

/**
 * Get the list of roles that have separate price fields enabled
 * @return Array
 * @since 1.4.8
 */
function bfwc_get_role_price_fields() {

	$roles = get_option( 'bfwc_role_price_fields', array() );
	return $roles;

}

/**
 * Whether bookings are created when a product is added to the cart or after the order is placed
 * Default is for dates to be blocked when the product is added to the cart
 * @since 2.0.0
 */
function bfwc_get_block_dates_after_order() {
	$block = get_option( 'bfwc_block_after_order', 'no' );
	return $block;
}

/**
 * Get expiry limit in minutes
 * @since 2.0.0
 */
function bfwc_get_cart_expiry() {
	$expiry = get_option( 'bfwc_cart_expiry', 30 );
	return $expiry;
}

/**
 * Automatically set the end date to the same as the start date
 * @since 2.0.0
 */
function bfwc_get_auto_set_end( $product_id ) {
	return apply_filters( 'bfwc_auto_set_end', 'no', $product_id );
}

/**
 * Checks if WooCommerce High-Performance Order Storage (HPOS) is enabled
 * @since 2.0.3
 */
function bfwc_hpos_enabled() {
	if ( OrderUtil::custom_orders_table_usage_is_enabled() ) {
		// HPOS usage is enabled.
		return true;
	} else {
		// Traditional CPT-based orders are in use.
		return false;
	}
}

/**
 * Allow discount rate to be increased, particularly if discount type is "fixed"
 * @since 2.0.5
 */
function bfwc_get_max_discount_rate( $product_id ) {
	$rate = intval( apply_filters( 'bfwc_discount_rate_max', 100, $product_id ) ); // force integer
	return $rate;
}
