<?php
/**
 * Functions for when the order is placed
 * @since 1.0.0
 * @package Bookings for WooCommerce
 */

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Check that the booking we're trying to buy is still valid
 * If it's not, remove it from the cart and display a notice
 * @since 1.2.1
 */
function bfwc_before_checkout_process() {
	$cart = WC()->cart->get_cart_contents();
	if( $cart ) {
		foreach( $cart as $item_key=>$values ) {
			// Check for a booking
			if( isset( $values['booking']['_booking_id'] ) ) {
				$booking_id = $values['booking']['_booking_id'];
				$status = get_post_meta( $booking_id, 'bfwc_status', true );
				// We can only buy a booking if it's pending
				if( $status != 'pending' ) {
					WC()->cart->remove_cart_item($item_key);
				}
			}
		}
	}
}
add_action( 'woocommerce_before_checkout_process', 'bfwc_before_checkout_process' );

/**
 * Add custom booking meta to order
 */
function bfwc_add_custom_data_to_order( $item, $cart_item_key, $values, $order ) {
	foreach( $item as $cart_item_key=>$values ) {
		if( isset( $values['booking'] ) ) {
			$item->add_meta_data( 'booking', $values['booking'], true );
		}
	}
}
add_action( 'woocommerce_checkout_create_order_line_item', 'bfwc_add_custom_data_to_order', 10, 4 );

/**
 * Add booking meta and set booking status to processing
 * @since 1.2.1
 */
function bfwc_set_booking_processing( $order_id ) {

	do_action( 'bfwc_start_set_booking_processing' );
	if( did_action( 'bfwc_start_set_booking_processing' ) > 1 ) {
		return;
	}

	// Avoid running twice
	$order = wc_get_order( $order_id );
	//$booking_id = get_post_meta( $order_id, 'bfwc_booking_id', true );
	$booking_id = $order->get_meta( 'bfwc_booking_id', true ); // WC HPOS compatibility
	if( $booking_id ) {
		return;
	}

	$payment_method = is_callable( array( $order, 'get_payment_method' ) ) ? $order->get_payment_method() : $order->payment_method;

	// Don't publish bookings for COD orders.
	if ( $order->has_status( 'processing' ) && 'cod' === $payment_method ) {
		//	return;
	}

	// Get the booking meta data and update the booking status
	$order_items = $order->get_items();
	if( $order_items ) {
		foreach( $order_items as $order_item ) {
			$booking_data = $order_item->get_meta( 'booking' );
			if( ! empty( $booking_data ) ) {
				// This item is a booking
				if( isset( $booking_data['_booking_id'] ) ) {
					// If the booking ID is already set, then dates were blocked at the add to cart stage
					$booking_id = absint( $booking_data['_booking_id'] );
				} else {
					// If there's no booking id, we need to create one now that the order has been placed
					$booking_id = bfwc_create_new_booking( $booking_data, $order_item->get_product_id() );
				}

				if( $booking_id && ! is_wp_error( $booking_id ) ) {
					update_post_meta( $booking_id, 'bfwc_status', 'processing' );
					// Update the order with the booking ID
					//update_post_meta( $order_id, 'bfwc_booking_id', $booking_id );
					$order->update_meta_data( 'bfwc_booking_id', $booking_id ); // WC HPOS compatibility
					update_post_meta( $booking_id, 'bfwc_user_id', $order->get_user_id() );
					update_post_meta( $booking_id, 'bfwc_user_email', $order->get_billing_email() );
					update_post_meta( $booking_id, 'bfwc_order_id', $order_id );
					if( ! empty( $booking_data['num_bookings'] ) ) {
						update_post_meta( $booking_id, 'bfwc_quantity', $booking_data['num_bookings'] );
					}
				}
			}
		}
		$order->save(); // save updated meta data to the database
	}
}
add_action( 'woocommerce_thankyou', 'bfwc_set_booking_processing', 10, 1 );
// add_action( 'woocommerce_order_status_processing', 'bfwc_set_booking_processing', 10, 1 );

/**
 * Update booking status when payment goes through
 */
function bfwc_set_booking_confirmed( $order_id ) {
	$order = wc_get_order( $order_id );
	// Get the booking meta data and update the booking status
	$order_items = $order->get_items();
	if( $order_items ) {
		foreach( $order_items as $order_item ) {
			$booking_data = $order_item->get_meta( 'booking' );
			if( isset( $booking_data['_booking_id'] ) ) {
				$booking_id = absint( $booking_data['_booking_id'] );
				update_post_meta( $booking_id, 'bfwc_status', 'confirmed' );
			} else {
				// Check if the booking ID is set from the order, not from the cart
				// Since 1.3.4
				//$booking_id = get_post_meta( $order_id, 'bfwc_booking_id', true );
				$booking_id = $order->get_meta( 'bfwc_booking_id', true ); // WC HPOS compatibility
				if( $booking_id ) {
					update_post_meta( $booking_id, 'bfwc_status', 'confirmed' );
				}
			}
		}
	}
}
add_action( 'woocommerce_order_status_completed', 'bfwc_set_booking_confirmed', 10, 1 );

/**
 * Add booking information to view order page
 */
function bfwc_order_item_name( $product_name, $item, $is_visible ) {
	$date_format = get_option( 'date_format' );
	if( isset( $item['booking'] ) ) {
		$product_name .= '<ul>';
		if( isset( $item['booking']['start_date_text'] ) ) {
			$product_name .= '<li>' . __( 'Booking Starts: ', 'bfwc' ) . $item['booking']['start_date_text'] . '</li>';
		}
		if( isset( $item['booking']['num_units'] ) ) {
			$product_name .= '<li>' . __( 'Number of Weeks: ', 'bfwc' ) . $item['booking']['num_units'] . '</li>';
		}
		if( isset( $item['booking']['end_date_text'] ) ) {
			$product_name .= '<li>' . __( 'Booking Ends: ', 'bfwc' ) . $item['booking']['end_date_text'] . '</li>';
		}
		if( isset( $item['booking']['num_bookings'] ) ) {
			$product_name .= sprintf(
				'<li>%s %s</li>',
				apply_filters( 'bfwc_quantity_bookings_text', __( 'Quantity:', 'bfwc' ) ),
				$item['booking']['num_bookings']
			);
		}
		$product_name .= '</ul>';
	}
	return $product_name;
}
add_filter( 'woocommerce_order_item_name', 'bfwc_order_item_name', 10, 3 );

/**
 * Remove quantity field
 */
add_filter( 'woocommerce_order_item_quantity_html', '__return_false' );
