<?php
/**
 * Functions for integrating with PEWC
 * We have to override some of the product price calculations so that we include the booking cost and the extras cost
 * Extras costs can include per_unit costs
 * @todo Change Bookings price calculation to JavaScript, away from AJAX, for consistency with PEWC
 * @since 1.1.0
 * @package Bookings for WooCommerce
 */

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}

// We don't want to load this if Product Extras isn't active
if( ! defined( 'PEWC_PLUGIN_VERSION' ) ) {
	return;
}

/**
 * Add a Price per unit option to product extra fields
 * @since 1.1.0
 */
function bfwc_add_per_unit_checkbox( $item, $group_key, $item_key ) { ?>
  <div class="pewc-per-unit product-extra-field-third">
		<?php $checked = ! empty( $item['field_per_unit'] ); ?>
		<input <?php checked( $checked, 1, true ); ?> type="checkbox" class="pewc-field-per-unit" name="_product_extra_groups_<?php echo esc_attr( $group_key ); ?>_<?php echo esc_attr( $item_key ); ?>[field_per_unit]" value="1">
		<label class="pewc-field-label"><?php _e( 'Price per booking unit?', 'bfwc' ); ?></label>
	</div>
  <?php
}
add_action( 'pewc_end_checkbox_row', 'bfwc_add_per_unit_checkbox', 10, 3 );

/**
 * Add a Price per unit option to a new product extra field
 * @since 1.1.0
 */
function bfwc_add_per_unit_checkbox_new_field() { ?>
	<div class="product-extra-field-third pewc-per-unit">
		<input type="checkbox" class="pewc-field-per-unit pewc-field-per_unit" name="" value="1">
		<label class="pewc-field-label"><?php _e( 'Price per booking unit?', 'bfwc' ); ?></label>
	</div>
  <?php
}
add_action( 'pewc_end_checkbox_row_new_item', 'bfwc_add_per_unit_checkbox_new_field' );

/**
 * Calculate price for a per unit option
 * @since 1.1.1
 */
function bfwc_filter_cart_item_data_price( $price, $cart_item_data, $item, $group_id, $field_id ) {
	if( ! empty( $item['field_per_unit'] ) && ! empty( $cart_item_data['booking']['num_units_int'] ) ) {
    $price = $price * absint( $cart_item_data['booking']['num_units_int'] );
	}
  return $price;
}
add_filter( 'pewc_filter_cart_item_data_price', 'bfwc_filter_cart_item_data_price', 10, 5 );

/**
 * Find the total calculated price, including booking and extras
 * Uses #pewc_total_calc_price hidden input field which is set in pewc.js
 * This overrides the default calculated_booking_cost set by Bookings bwfc_add_cart_item_data
 * @param	Numeric	$cost
 * @param Array $post_obj	$_POST object
 * @return Numeric
 * @since 1.1.1
 */
function bfwc_pewc_filter_calculated_booking_cost( $cost, $post_obj ) {
	if( isset( $post_obj['pewc_total_calc_price'] ) ) {
		$cost = $post_obj['pewc_total_calc_price'];
	}
	return $cost;
}
add_filter( 'bfwc_filter_calculated_booking_cost', 'bfwc_pewc_filter_calculated_booking_cost', 10, 2 );

/**
 * Also filter calculated product total price
 * If product type is booking
 * Overrides price set by PEWC
 * @param	Numeric	$price
 * @param Array $value	Cart object
 * @param Numeric $key 	Cart item key
 * @return Numeric
 * @since 1.1.3
 */
function bfwc_pewc_filter_calculated_cost_before_calculate_totals( $price, $value, $key='' ) {
	$product_id = $value['product_id'];
	$product = wc_get_product( $product_id );
	if( $product->get_type() == 'simple_booking' ) {
		if( isset( $value['booking']['cost'] ) ) {
			$price = floatval( $value['booking']['cost'] );
		}
	}
	return $price;
}
add_filter( 'pewc_filter_calculated_cost_before_calculate_totals', 'bfwc_pewc_filter_calculated_cost_before_calculate_totals', 10, 2 );

/**
 * Filter additional cart items
 * @since 1.1.0
 */
function bfwc_end_add_cart_item_data( $cart_item_data, $item, $group_id, $field_id ) {
  if( ! empty( $item['per_unit'] ) ) {
		$cart_item_data['product_extras']['groups'][$group_id][$field_id]['per_unit'] = 1;
	}
}
add_action( 'pewc_end_add_cart_item_data', 'bfwc_end_add_cart_item_data', 10, 4 );

/**
 * Filter single product classes
 * @since 1.1.0
 */
function bfwc_filter_pewc_single_product_classes( $classes, $item ) {
  if( ! empty( $item['field_per_unit'] ) ) {
  	$classes[] = 'pewc-per-unit-pricing';
  }
  return $classes;
}
add_filter( 'pewc_filter_single_product_classes', 'bfwc_filter_pewc_single_product_classes', 10, 2 );

/**
 * Filter the option label
 * @since 1.1.0
 */
function bfwc_filter_field_label( $label, $item, $id ) {
  if( isset( $item['field_per_unit'] ) ) {
		$label = '<label for="' . esc_attr( $id ) . '">';
  	if( isset( $item['field_label'] ) || isset( $item['field_price'] ) ) {
  		if( isset( $item['field_label'] ) ) {
  			$label .= esc_html( $item['field_label'] );
  		}
  		$label .= '<span class="required"> &#42;</span>';
  		if( ! empty( $item['field_price'] ) && $item['field_type'] != 'name_price' ) {
  			$formatted_price = pewc_wc_format_price( $item['field_price'] );
  			$label .= ' <span class="pewc-field-price">' . $formatted_price;
          $unit_label = bfwc_get_unit_label( get_the_ID() );
					if( $unit_label ) {
						$label .= ' <span class="pewc-per-unit-label">' . __( 'per ', 'bfwc' ) . strtolower( esc_html( $unit_label['single'] ) ) . '</span>';
					}
  			$label .= '</span>';
  		}
  	}
		$label .= '</label>';
  }
  return $label;
}
add_filter( 'pewc_filter_field_label', 'bfwc_filter_field_label', 10, 3 );

function bfwc_pewc_item_params( $params, $field_id ) {
	$params[] = 'field_per_unit';
	return $params;
}
add_filter( 'pewc_item_params', 'bfwc_pewc_item_params', 10, 2 );

/**
 * Filter the product price - ensure that Add-Ons Ultimate price does not override the Bookings price
 */
function bfwc_pewc_price_with_extras_before_calc_totals( $price, $values ) {
	if( ! empty( $values['booking']['cost'] ) ) {
		return $values['booking']['cost'];
	}
	return $price;
}
add_filter( 'pewc_price_with_extras_before_calc_totals', 'bfwc_pewc_price_with_extras_before_calc_totals', 10, 2 );
