<?php
/**
 * Functions for searching availability of bookable products
 * @since 2.0.0
 * @package Bookings for WooCommerce
 */

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Search for availability on all bookable products between two dates
 * @since 2.0.0
 */
function bfwc_query_products_by_date( $q ) {

	// We need to find all products which are available between these dates
	$start_date = isset( $_GET['start_ymd'] ) ? $_GET['start_ymd'] : false;
	$end_date = isset( $_GET['end_ymd'] ) ? $_GET['end_ymd'] : false;

	if( ! $start_date || ! $end_date ) {
		return $q;
	}

	$available_products = bfwc_get_all_bookable_products();

	// Iterate through each day and find availability per product
	$start_date = new DateTime( $start_date );
	$start_date_ymd = $start_date->format( 'Y-m-d' );
	$end_date = new DateTime( $end_date );
	$end_date_ymd = $end_date->format( 'Y-m-d' );

	// Iterate through each date from start to end
	// Use $check_date then add one day
	$check_date = new DateTime( $start_date_ymd );
	$check_date = $check_date->format( 'Y-m-d' );

	do {

		$check_date = new DateTime( $check_date );
		$date = $check_date->format( 'Y-m-d' );

		$day = bfwc_get_page_by_title( $date, 'bfwc_booked_days' );
		if( $day ) {

			$day_id = $day->ID;
			// Get the booking meta for this day
			$booking_meta = get_post_meta( $day_id, 'booking_meta', true );
			if( $booking_meta ) {

				// Now iterate through each product booked on this day
				foreach( $booking_meta as $product_id=>$product_data ) {

					$max_bookings = bfwc_get_max_bookings( $product_id );
					$used_bookings = $product_data['used_bookings'];
					if( $used_bookings >= $max_bookings ) {

						// This product has no availability on this date, so we can remove it from our list of products to return
						if( ( $key = array_search( $product_id, $available_products) ) !== false) {
							unset( $available_products[$key] );
							continue;
						}

					}

				}

			}

		} else {

			// This day doesn't have any bookings

		}

		// Move on to the next day
		$check_date->modify( '+1 day' );
		$check_date = $check_date->format( 'Y-m-d' );

	} while( $check_date <= $end_date_ymd );

	$q->set( 'post__in', $available_products );

}
add_action( 'woocommerce_product_query', 'bfwc_query_products_by_date' );

/**
 * Query all bookable products and save array of IDs in transient
 * @since 2.0.0
 */
function bfwc_get_all_bookable_products() {

	$bookable_products = get_transient( 'bfwc_bookable_products', array() );

	if( ! $bookable_products ) {

		// Query all bookable products
		$args = array(
			'type' 				=> 'simple_booking',
			'post_status'		=> 'publish',
			'limit'				=> -1,
			'return'			=> 'ids'
		);
		$bookable_products = wc_get_products( $args );

		set_transient( 'bfwc_bookable_products', $bookable_products );

	}

	return $bookable_products;

}

/**
 * Query all bookable products and save array of IDs in transient
 * @since 2.0.0
 */
function bfwc_get_bookable_products_titles() {

	$bookable_products = bfwc_get_all_bookable_products();
	$titles = array();

	if( $bookable_products ) {

		foreach( $bookable_products as $product_id ) {

			$titles[$product_id] = get_the_title( $product_id );

		}

	}

	return $titles;

}

/**
 * Ensure we pass start and end params to the search URL
 * @since 2.0.0
 */
function bfwc_redirect_search() {

	$start_ymd = ! empty( $_POST['search_start_date_ymd'] ) ? $_POST['search_start_date_ymd'] : false;
	$end_ymd = ! empty( $_POST['search_end_date_ymd'] ) ? $_POST['search_end_date_ymd'] : false;

	if( $start_ymd && $end_ymd ) {

		global $wp;
		$url = home_url( $wp->request );
		wp_safe_redirect(
			add_query_arg(
				array(
					'start_ymd' => $start_ymd,
					'end_ymd'   => $end_ymd
				),
				$url
			)
		);

	}

}
add_action( 'template_redirect', 'bfwc_redirect_search' );

/**
 * Generate a search form
 * @since 2.0.0
 */
function bfwc_search_form() {

	if( bfwc_get_archive_search() == 'yes' ) {

		wc_get_template( 'templates/booking-search-form.php', array(), 'bfwc', BFWC_PLUGIN_DIR_PATH );
		
	}

}
add_action( 'woocommerce_before_shop_loop', 'bfwc_search_form', 999 );
